/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef _NT4GA_ADAPTER_H_
#define _NT4GA_ADAPTER_H_

#include "common_adapter_defs.h"

struct AdapterInfo_s;

/*
 * DN-0060 section 9
 */
typedef struct HwInfo_s
{
  /* pciids */
  uint16_t pci_vendor_id;
  uint16_t pci_device_id;
  uint16_t pci_sub_vendor_id;
  uint16_t pci_sub_device_id;
  uint16_t pci_class_id;

  /* Derived from pciid */
  NthwAdapterId_t nNthwAdapterId;
  int hw_platform_id;
  int hw_product_type;
  int hw_reserved1;
} HwInfo_t;

/*
 * Services provided by the adapter module
 */
#include "nt4ga_pci_ta_tg.h"
#include "nt4ga_filter.h"
#include "nt4ga_stat.h"
#include "nt4ga_tfg.h"
#include "nt4ga_link.h"

#include "sensors.h"
#include "i2c_nim.h"
#include "sensor_types.h"

typedef struct AdapterInfo_s
{
  struct Nt4gaPciTaTg_s nt4ga_pci_ta_tg;
  struct Nt4gaStat_s nt4ga_stat;
  struct Nt4gaFilter_s nt4ga_filter;
  struct Nt4gaTfg_s nt4ga_tfg;
  struct Nt4gaLink_s nt4ga_link;

  struct NthwMcu* mpNthwMcu;

  struct HwInfo_s hw_info;
  struct FpgaInfo_s fpga_info;

  uint16_t adapter_sensors_cnt;
  uint16_t nim_sensors_cnt[NUM_ADAPTER_PORTS_MAX];
  struct nt_sensor_group* adapter_sensors;
  struct nim_sensor_group* nim_sensors[NUM_ADAPTER_PORTS_MAX];

  char* mpPortIdStr[NUM_ADAPTER_PORTS_MAX];
  char* mpAdapterIdStr;
  char* pDevName;
  volatile bool* pbShutdown;

  int adapterNo;
  int nRxHostBuffers;
  int nTxHostBuffers;
} AdapterInfo_t;

/*
 * Monitor task operations.  This structure defines the management hooks for
 * Napatech network devices.  The following hooks can be defined; unless noted
 * otherwise, they are optional and can be filled with a null pointer.
 *
 * int (*mto_open)(int adapter, int port);
 *     The function to call when a network device transitions to the up state,
 *     e.g., `ip link set <interface> up`.
 *
 * int (*mto_stop)(int adapter, int port);
 *     The function to call when a network device transitions to the down state,
 *     e.g., `ip link set <interface> down`.
 */
struct monitor_task_ops {
  int (*mto_open)(int adapter, int port);
  int (*mto_stop)(int adapter, int port);
};

#include <pthread.h>
#include <signal.h>

/* The file nt4ga_adapter.c defines the next four variables. */
extern pthread_t monitor_tasks[NUM_ADAPTER_MAX];
extern volatile int monitor_task_is_running[NUM_ADAPTER_MAX];

/*
 * Function that sets up signal handler(s) that stop the monitoring tasks.
 */
int set_up_signal_handlers_to_stop_monitoring_tasks(void);

int Nt4ga_Adapter_init(struct AdapterInfo_s* pAdapterInfo);
int Nt4ga_Adapter_deinit(struct AdapterInfo_s* pAdapterInfo);

int Nt4ga_Adapter_Status(struct AdapterInfo_s* pAdapterInfo);
int Nt4ga_Adapter_Transmit_Packet(struct AdapterInfo_s* pAdapterInfo, int nIntfNo, uint8_t* pPkt, int nPktLen);

int Nt4ga_Adapter_show_info(struct AdapterInfo_s* pAdapterInfo, FILE* pfh);

/* SPI for sensors reading */
NthwSpis_t* new_sensors_t_spi(struct nthw_fpga_s* pFpga);

#endif /* _NT4GA_ADAPTER_H_ */
