/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __NTHW_FPGA_RST_NT50B0X_H__
#define __NTHW_FPGA_RST_NT50B0X_H__

#include "nthw_drv.h"

#include "nthw_fpga_model.h"

#include "nthw_rac.h"
#include "nthw_iic.h"

struct NthwFpga_rst_nt50b0x
{
  nthw_fpga_t* mpFpga;

  int mnFpgaProductId;
  int mnFpgaVersion;
  int mnFpgaRevision;

  int mnHwId;

  int mnSiLabsClockSynthModel;
  uint8_t mnSiLabsClockSynthI2cAddr;

  nthw_module_t* mpModRst;
  int mnInstance;

  /* PRM/PCM: */
  nthw_module_t* mpModPrm;
  nthw_module_t* mpModPcm;

  nthw_field_t* mpFldRstSys;
  nthw_field_t* mpFldRstSysMmcm;
  nthw_field_t* mpFldRstCoreMmcm;
  nthw_field_t* mpFldRstRpp;
  nthw_field_t* mpFldRstDdr4;
  nthw_field_t* mpFldRstSdc;
  nthw_field_t* mpFldRstPhy;
  nthw_field_t* mpFldRstSerdesRx;
  nthw_field_t* mpFldRstSerdesTx;
  nthw_field_t* mpFldRstSerdesRxDatapath;
  nthw_field_t* mpFldRstPcsRx;
  nthw_field_t* mpFldRstMacRx;
  nthw_field_t* mpFldRstMacTx;
  nthw_field_t* mpFldRstPtp;
  nthw_field_t* mpFldRstTs;
  nthw_field_t* mpFldRstPtpMmcm;
  nthw_field_t* mpFldRstTsMmcm;
  nthw_field_t* mpFldRstPeriph;
  nthw_field_t* mpFldRstTsmRefMmcm;
  nthw_field_t* mpFldRstTmc;

  /* CTRL register field pointers */
  nthw_field_t* mpFldCtrlTsClkSelOverride;
  nthw_field_t* mpFldCtrlTsClkSel;
  nthw_field_t* mpFldCtrlTsClkSelRef;
  nthw_field_t* mpFldCtrlPtpMmcmClkSel;

  /* STAT register field pointers */
  nthw_field_t* mpFldStatDdr4MmcmLocked;
  nthw_field_t* mpFldStatSysMmcmLocked;
  nthw_field_t* mpFldStatCoreMmcmLocked;
  nthw_field_t* mpFldStatDdr4PllLocked;
  nthw_field_t* mpFldStatPtpMmcmLocked;
  nthw_field_t* mpFldStatTsMmcmLocked;
  nthw_field_t* mpFldStatTsmRefMmcmLocked;
  /* STAT register field pointers nt50b0x */
  nthw_field_t* mpFldStatDdr4CalibComplete;

  /* LATCH register field pointers */
  nthw_field_t* mpFldLatchDdr4CalibComplete;

  /* STICKY register field pointers */
  nthw_field_t* mpFldStickyPtpMmcmUnlocked;
  nthw_field_t* mpFldStickyTsMmcmUnlocked;
  nthw_field_t* mpFldStickyDdr4MmcmUnlocked;
  nthw_field_t* mpFldStickyDdr4PllUnlocked;
  nthw_field_t* mpFldStickyCoreMmcmUnlocked;
  nthw_field_t* mpFldStickyPciSysMmcmUnlocked;
  nthw_field_t* mpFldStickyTsmRefMmcmUnlocked;

  /* POWER register field pointers */
  nthw_field_t* mpFldPowerPuPhy;
  nthw_field_t* mpFldPowerPuNseb;
  /*  */

  /* RST extra fields */
  nthw_field_t* mpFldStatPhyRdy;
  nthw_field_t* mpFldLatchPhyRdy;

  /* PRM extra fields */
  nthw_field_t* mpFldRstPlatform; /* PRM */

  /* PCM extra fields */
  nthw_field_t* mpFldCtrlTsMmcmRst; /* PCM */
  nthw_field_t* mpFldLatchTsMmcmLocked; /* PCM */
};

int NthwFpga_rst_nt50b0x_init(struct FpgaInfo_s* pFpgaInfo, struct NthwFpga_rst_nt50b0x* pRst);
int NthwFpga_rst_nt50b0x_reset(nthw_fpga_t* pFpga, const struct NthwFpga_rst_nt50b0x* p);

int NthwFpga_rst_nt50b0x_wait_sdc_calibrated(nthw_fpga_t* pFpga, const struct NthwFpga_rst_nt50b0x* p);
int NthwFpga_rst_nt50b0x_wait_ddr4_calibration_complete(nthw_fpga_t* pFpga, const struct NthwFpga_rst_nt50b0x* p);

int NthwFpga_rst_nt50b0x_2X25_resetPhyMac(nthw_fpga_t* pFpga, const struct NthwFpga_rst_nt50b0x* p);

#endif /* __NTHW_FPGA_RST_NT50B0X_H__ */
