/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __NTHW_HIF_H__
#define __NTHW_HIF_H__

#define NTHW_TG_CNT_SIZE (4ULL)

struct NthwHif
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModHif;
  int mnInstance;

  nthw_register_t* mpRegProdIdLsb;
  nthw_field_t* mpFldProdIdLsbRevId;
  nthw_field_t* mpFldProdIdLsbVerId;
  nthw_field_t* mpFldProdIdLsbGroupId;

  nthw_register_t* mpRegProdIdMsb;
  nthw_field_t* mpFldProdIdMsbTypeId;
  nthw_field_t* mpFldProdIdMsbBuildNo;

  nthw_register_t* mpRegBuildTime;
  nthw_field_t* mpFldBuildTime;

  nthw_register_t* mpRegBuildSeed;
  nthw_field_t* mpFldBuildSeed;

  nthw_register_t* mpRegCoreSpeed;
  nthw_field_t* mpFldCoreSpeed;
  nthw_field_t* mpFldDdr3Speed;

  nthw_register_t* mpRegIntMask;
  nthw_field_t* mpFldIntMaskTimer;
  nthw_field_t* mpFldIntMaskPort;
  nthw_field_t* mpFldIntMaskPps;

  nthw_register_t* mpRegIntClr;
  nthw_field_t* mpFldIntClrTimer;
  nthw_field_t* mpFldIntClrPort;
  nthw_field_t* mpFldIntClrPps;

  nthw_register_t* mpRegIntForce;
  nthw_field_t* mpFldIntForceTimer;
  nthw_field_t* mpFldIntForcePort;
  nthw_field_t* mpFldIntForcePps;

  nthw_register_t* mpRegSampleTime;
  nthw_field_t* mpFldSampleTime;

  nthw_register_t* mpRegStatus;
  nthw_field_t* mpFldStatusTagsInUse;
  nthw_field_t* mpFldStatusWrErr;
  nthw_field_t* mpFldStatusRdErr;

  nthw_register_t* mpRegStatCtrl;
  nthw_field_t* mpFldStatCtrlEna;
  nthw_field_t* mpFldStatCtrlReq;

  nthw_register_t* mpRegStatRx;
  nthw_field_t* mpFldStatRxCounter;

  nthw_register_t* mpRegStatTx;
  nthw_field_t* mpFldStatTxCounter;

  nthw_register_t* mpRegStatRefClk;
  nthw_field_t* mpFldStatRefClkRefClk;

  nthw_register_t* mpRegPciTest0;
  nthw_field_t* mpFldPciTest0;

  nthw_register_t* mpRegPciTest1;
  nthw_field_t* mpFldPciTest1;

  nthw_register_t* mpRegPciTest2;
  nthw_field_t* mpFldPciTest2;

  nthw_register_t* mpRegPciTest3;
  nthw_field_t* mpFldPciTest3;

  nthw_register_t* mpRegConfig;
  nthw_field_t* mpFldMaxTlp;
  nthw_field_t* mpFldMaxRead;
  nthw_field_t* mpFldExtTag;

  int mnFpgaIdItem;
  int mnFpgaIdProd;
  int mnFpgaIdVer;
  int mnFpgaIdRev;
  int mnFpgaIdBuildNo;

  int mnFpgaParamHifPerPs;
  uint32_t mnFpgaHifRefClkFreq;
};


typedef struct NthwHif NthwHif_t;
typedef struct NthwHif NthwHif;

struct NthwHif_EndPointErrCounters {
  uint32_t nErrCorrectable, nErrNonFatal, nErrFatal;
};

struct NthwHif_EndPointCounters {
  int nNumaNode;

  int nTgDirection;
  int nTgPktSize;
  int nTgNumPkts;
  int nTgDelay;

  uint64_t curRx, curTx;
  uint64_t curPciNtUtil, curPciXilUtil;
  uint64_t nRefClkCnt;

  uint64_t nTagsInUse;
  uint64_t nRdErr;
  uint64_t nWrErr;

  struct NthwHif_EndPointErrCounters sRcEpPre, sRcEpPost, sRcEpDelta;
  struct NthwHif_EndPointErrCounters sEpRcPre, sEpRcPost, sEpRcDelta;

  int boError;
};

struct NthwHif_EndPoints {
  struct NthwHif_EndPointCounters pri, sla;
};

NthwHif_t* NthwHif_new(void);
void NthwHif_delete(NthwHif_t* p);
int NthwHif_init(NthwHif_t* p, nthw_fpga_t* pFpga, int nInstance);

int NthwHif_setupConfig(NthwHif_t* p);

int NthwHif_trigger_sample_time(NthwHif_t* p);

int NthwHif_StatReqEnable(NthwHif_t* p);
int NthwHif_StatReqDisable(NthwHif_t* p);
int NthwHif_StatSample(NthwHif_t* p, uint64_t* pRxRate, uint64_t* pTxRate, uint64_t* pRefClkCnt, uint64_t* pTagsInUse, uint64_t* pRdErrCnt, uint64_t* pWrErrCnt);

int NthwHif_getStat(NthwHif_t* p, uint32_t* pRxCnt, uint32_t* pTxCnt, uint32_t* pRefClkCnt, uint32_t* pTgUnitSize, uint32_t* pTgRefFreq, uint64_t* pTagsInUse, uint64_t* pRdErr, uint64_t* pWrErr);
int NthwHif_getStatRate(NthwHif_t* p, uint64_t* pPciRxRate, uint64_t* pPciTxRate, uint64_t* pRefClkCnt, uint64_t* pTagsInUse, uint64_t* pRdErrCnt, uint64_t* pWrErrCnt);

int NthwHif_EndPointCountersSample(NthwHif_t* p, struct NthwHif_EndPointCounters* epc);

#endif /* __NTHW_HIF_H__ */
