/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_MAC_TFG_H_
#define NTHW_MAC_TFG_H_

struct NthwMacTfg
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModMacTfg;
  int mnInstance;

  /* Params */
  int mnIfgSpeedMul;
  int mnIfgSpeedDiv;

  /* TFG */
  nthw_register_t* mpRegTfgData;
  nthw_register_t* mpRegTfgAddr;
  nthw_register_t* mpRegTfgCtrl;
  nthw_register_t* mpRegTfgRepetition;

  /* TFG ADDR */
  nthw_field_t* mpFldTfgAddrWriteRAMAdr;
  nthw_field_t* mpFldTfgAddrReadEnable;
  nthw_field_t* mpFldTfgAddrReadDone;

  /* TFG DATA */
  nthw_field_t* mpFldTfgDataLength;
  nthw_field_t* mpFldTfgDataGap;
  nthw_field_t* mpFldTfgDataId;

  /* TFG CTRL */
  nthw_field_t* mpFldTfgWrap;
  nthw_field_t* mpFldTfgRestart;
  nthw_field_t* mpFldTfgEnable;
  nthw_field_t* mpFldTfgTimeMode;
  nthw_field_t* mpFldTfgIdPos;
  nthw_field_t* mpFldTfgIdEna;
  nthw_field_t* mpFldTfgTxActive;

  /* TFG REPETITION */
  nthw_field_t* mpFldTfgRepetitionCount;

};

typedef struct NthwMacTfg NthwMacTfg_t;
typedef struct NthwMacTfg NthwMacTfg;

NthwMacTfg_t* NthwMacTfg_new(void);
void NthwMacTfg_delete(NthwMacTfg_t* p);
int NthwMacTfg_init(NthwMacTfg_t *p, nthw_fpga_t* pFpga, int nInstance);

void NthwMacTfg_TfgTxStart(NthwMacTfg_t* p, uint32_t repetition, uint32_t size, uint32_t pkt_gap);
void NthwMacTfg_TfgTxStop(NthwMacTfg_t* p);

bool NthwMacTfg_isTfgTransmitting(NthwMacTfg_t* p);

double NthwMacTfg_getTfgRateFactor(NthwMacTfg_t* p);
double NthwMacTfg_getTfgGapFactor(NthwMacTfg_t* p);

#endif /* NTHW_MAC_TFG_H_ */
