/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_RBH_H_
#define NTHW_RBH_H_

struct NthwRbh
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModRbh;
  int mnInstance;

  int mnModuleMajorVersion;
  int mnModuleMinorVersion;

  uint64_t mnModuleVersionPacked64;

  nthw_register_t* mpRegCtrl;
  nthw_field_t* mpFldCtrlEnableQmh;
  nthw_field_t* mpFldCtrlEnablePM;
  nthw_field_t* mpFldCtrlEnableMetaData;
  nthw_field_t* mpFldCtrlReserved;
  nthw_field_t* mpFldCtrlHbMax;
  nthw_field_t* mpFldCtrlHbSegmentSize;

  nthw_register_t* mpRegFlushAdrHi;
  nthw_field_t* mpFldFlushAdrHiValue;

  nthw_register_t* mpRegFlushAdrLo;
  nthw_field_t* mpFldFlushAdrLoValue;

  nthw_register_t* mpRegStatus;
  nthw_field_t* mpFldStatusPwaDataFifoEmpty;
  nthw_field_t* mpFldStatusBufferEmpty;

  nthw_register_t* mpRegDebReg1;
  nthw_field_t* mpFldDebReg1Value;

  nthw_register_t* mpRegDebReg2;
  nthw_field_t* mpFldDebReg2Value;

  nthw_register_t* mpRegDebReg3;
  nthw_field_t* mpFldDebReg3Value;

  nthw_register_t* mpRegHostBufSizeMemCtrl;
  nthw_field_t* mpFldHostBufSizeMemCtrlAdr;
  nthw_field_t* mpFldHostBufSizeMemCtrlCnt;

  nthw_register_t* mpRegHostBufSizeMemData;
  nthw_field_t* mpFldHostBufSizeMemDataHbSegmentCnt;

  nthw_register_t* mpRegWpSetMemCtrl;
  nthw_field_t* mpFldWpSetMemCtrlAdr;
  nthw_field_t* mpFldWpSetMemCtrlCnt;

  nthw_register_t* mpRegWpSetMemData;
  nthw_field_t* mpFldWpSetMemDataWp;
};

typedef struct NthwRbh NthwRbh_t;
typedef struct NthwRbh NthwRbh;

NthwRbh_t* NthwRbh_new(void);
void NthwRbh_delete(NthwRbh_t* p);
int NthwRbh_init(NthwRbh_t *p, nthw_fpga_t* pFpga, int nInstance);

void NthwRbh_setEnableQmh(NthwRbh_t* p, uint32_t enable);
void NthwRbh_setEnablePm(NthwRbh_t* p, uint32_t enable);
void NthwRbh_setEnableMetaData(NthwRbh_t* p, uint32_t enable);

void NthwRbh_setHostBufMax(NthwRbh_t* p, uint32_t hbMax);
void NthwRbh_setHostBufSegmentSize(NthwRbh_t* p, uint32_t hbSegSize);

void NthwRbh_setHostBufSegmentCount(NthwRbh_t* p, uint16_t addr, uint32_t hbSegCnt);
void NthwRbh_setWp(NthwRbh_t* p, uint16_t addr, uint64_t wpOffset);
void NthwRbh_setFlushAddr(NthwRbh_t* p, uint64_t flushAddr);

uint32_t NthwRbh_getStatusBufferEmpty(NthwRbh_t* p);
uint32_t NthwRbh_getStatusPwaDataFifoEmpty(NthwRbh_t* p);
uint32_t NthwRbh_getDeb1Value(NthwRbh_t* p);
uint32_t NthwRbh_getDeb2Value(NthwRbh_t* p);
uint32_t NthwRbh_getDeb3Value(NthwRbh_t* p);

#endif /* NTHW_RBH_H_ */
