/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __NTHW_SPIS_H__
#define __NTHW_SPIS_H__

struct NthwSpis
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModSpis;
  int mnInstance;

  nthw_register_t* mpReg_srr;
  nthw_field_t* mpFld_srr_rst;

  nthw_register_t* mpReg_cr;
  nthw_field_t* mpFld_cr_loop;
  nthw_field_t* mpFld_cr_en;
  nthw_field_t* mpFld_cr_txrst;
  nthw_field_t* mpFld_cr_rxrst;
  nthw_field_t* mpFld_cr_debug;

  nthw_register_t* mpReg_sr;
  nthw_field_t* mpFld_sr_done;
  nthw_field_t* mpFld_sr_txempty;
  nthw_field_t* mpFld_sr_rxempty;
  nthw_field_t* mpFld_sr_txfull;
  nthw_field_t* mpFld_sr_rxfull;
  nthw_field_t* mpFld_sr_txlvl;
  nthw_field_t* mpFld_sr_rxlvl;
  nthw_field_t* mpFld_sr_frame_err;
  nthw_field_t* mpFld_sr_read_err;
  nthw_field_t* mpFld_sr_write_err;

  nthw_register_t* mpReg_dtr;
  nthw_field_t* mpFld_dtr_dtr;

  nthw_register_t* mpReg_drr;
  nthw_field_t* mpFld_drr_drr;

  nthw_register_t* mpReg_ram_ctrl;
  nthw_field_t* mpFld_ram_ctrl_adr;
  nthw_field_t* mpFld_ram_ctrl_cnt;

  nthw_register_t* mpReg_ram_data;
  nthw_field_t* mpFld_ram_data_data;
};

typedef struct NthwSpis NthwSpis_t;
typedef struct NthwSpis NthwSpis;

NthwSpis_t* NthwSpis_new(void);
int NthwSpis_init(NthwSpis_t* p, nthw_fpga_t* pFpga, int nInstance);
void NthwSpis_delete(NthwSpis_t* p);

uint32_t NthwSpis_reset(NthwSpis_t* p);
uint32_t NthwSpis_enable(NthwSpis_t* p, bool bEnable);
uint32_t NthwSpis_get_rx_fifo_empty(NthwSpis_t* p, bool* pbEmpty);
uint32_t NthwSpis_read_rx_fifo(NthwSpis_t* p, uint32_t* pData);
uint32_t NthwSpis_read_sensor(NthwSpis_t* p, uint8_t nResultIdx, uint32_t* pSensorResult);

#endif /* __NTHW_SPIS_H__ */
