/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_HFU_H__
#define __FLOW_NTHW_HFU_H__

#include <stdint.h>
#include "nthw_fpga_model.h"

struct HfuNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;

  nthw_module_t *mHfu;

  nthw_register_t *mpRcpCtrl;
  nthw_field_t *mpRcpAddr;
  nthw_field_t *mpRcpCnt;

  nthw_register_t *mpRcpData;
  nthw_field_t *mpRcpDataLenAWr;
  nthw_field_t *mpRcpDataLenAOl4len;
  nthw_field_t *mpRcpDataLenAPosDyn;
  nthw_field_t *mpRcpDataLenAPosOfs;
  nthw_field_t *mpRcpDataLenAAddDyn;
  nthw_field_t *mpRcpDataLenAAddOfs;
  nthw_field_t *mpRcpDataLenASubDyn;
  nthw_field_t *mpRcpDataLenBWr;
  nthw_field_t *mpRcpDataLenBPosDyn;
  nthw_field_t *mpRcpDataLenBPosOfs;
  nthw_field_t *mpRcpDataLenBAddDyn;
  nthw_field_t *mpRcpDataLenBAddOfs;
  nthw_field_t *mpRcpDataLenBSubDyn;
  nthw_field_t *mpRcpDataLenCWr;
  nthw_field_t *mpRcpDataLenCPosDyn;
  nthw_field_t *mpRcpDataLenCPosOfs;
  nthw_field_t *mpRcpDataLenCAddDyn;
  nthw_field_t *mpRcpDataLenCAddOfs;
  nthw_field_t *mpRcpDataLenCSubDyn;
  nthw_field_t *mpRcpDataTtlWr;
  nthw_field_t *mpRcpDataTtlPosDyn;
  nthw_field_t *mpRcpDataTtlPosOfs;
  nthw_field_t *mpRcpDataCsinf;
  nthw_field_t *mpRcpDataL3prt;
  nthw_field_t *mpRcpDataL3frag;
  nthw_field_t *mpRcpDataTunnel;
  nthw_field_t *mpRcpDataL4prt;
  nthw_field_t *mpRcpDataOl3ofs;
  nthw_field_t *mpRcpDataOl4ofs;
  nthw_field_t *mpRcpDataIl3ofs;
  nthw_field_t *mpRcpDataIl4ofs;
};

struct HfuNthw *HfuNthw_new(void);
void HfuNthw_delete(struct HfuNthw *p);
int HfuNthw_init(struct HfuNthw *p, nthw_fpga_t *pFpga, int nInstance);

int HfuNthw_setup(struct HfuNthw *p, int nIdx, int nIdxCnt);
void hfuNthw_setDebugMode(struct HfuNthw *p, unsigned int nDebugMode);

#if 0
  uint32_t GetVersionMajor(void) {return (mHfu.getVersion()).getMajorVersion();}
  uint32_t GetVersionMinor(void) {return (mHfu.getVersion()).getMinorVersion();}
  void SetDebugOutput(int on) {if (on) mHfu.setDebugMode(Register::ON_WRITE); else mHfu.setDebugMode(Register::NO_DEBUG);}
#endif

/* RCP */
void HfuNthw_RcpSelect(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpCnt(const struct HfuNthw *p, uint32_t val);

void HfuNthw_RcpLenAWr(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenAOl4len(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenAPosDyn(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenAPosOfs(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenAAddDyn(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenAAddOfs(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenASubDyn(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenBWr(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenBPosDyn(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenBPosOfs(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenBAddDyn(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenBAddOfs(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenBSubDyn(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenCWr(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenCPosDyn(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenCPosOfs(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenCAddDyn(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenCAddOfs(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpLenCSubDyn(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpTtlWr(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpTtlPosDyn(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpTtlPosOfs(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpCsinf(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpL3prt(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpL3frag(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpTunnel(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpL4prt(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpOl3ofs(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpOl4ofs(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpIl3ofs(const struct HfuNthw *p, uint32_t val);
void HfuNthw_RcpIl4ofs(const struct HfuNthw *p, uint32_t val);

void HfuNthw_RcpFlush(const struct HfuNthw *p);

#endif /* __FLOW_NTHW_HFU_H__ */
