/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_HSH_H__
#define __FLOW_NTHW_HSH_H__

#include <stdint.h> /* uint32_t */
#include "nthw_fpga_model.h"

struct HshNthw;

typedef struct HshNthw HshNthw_t;
#if 0
typedef struct HshNthw HshNthw;
#endif

struct HshNthw *HshNthw_new(void);
void HshNthw_delete(struct HshNthw *p);
int HshNthw_init(struct HshNthw *p, nthw_fpga_t *pFpga, int nInstance);

int HshNthw_setup(struct HshNthw *p, int nIdx, int nIdxCnt);
void hshNthw_setDebugMode(struct HshNthw *p, unsigned int nDebugMode);

#if 0
uint32_t GetVersionMajor(void) {return (mHsh.getVersion()).getMajorVersion();}
uint32_t GetVersionMinor(void) {return (mHsh.getVersion()).getMinorVersion();}
void SetDebugOutput(int on) {if (on) mHsh.setDebugMode(Register::ON_WRITE); else mHsh.setDebugMode(Register::NO_DEBUG);}
#endif

/* RCP */
void HshNthw_RcpSelect(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpCnt(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpLoadDistType(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpMacPortMask(const struct HshNthw *p, uint32_t *val);
void HshNthw_RcpSort(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpQw0Pe(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpQw0Ofs(const struct HshNthw *p, int32_t val);
void HshNthw_RcpQw4Pe(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpQw4Ofs(const struct HshNthw *p, int32_t val);
void HshNthw_RcpW8Pe(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpW8Ofs(const struct HshNthw *p, int32_t val);
void HshNthw_RcpW8Sort(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpW9Pe(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpW9Ofs(const struct HshNthw *p, int32_t val);
void HshNthw_RcpW9Sort(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpW9P(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpPMask(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpWordMask(const struct HshNthw *p, uint32_t *val);
void HshNthw_RcpSeed(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpTnlP(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpHshValid(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpHshType(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpAutoIpv4Mask(const struct HshNthw *p, uint32_t val);
void HshNthw_RcpFlush(const struct HshNthw *p);

struct HshNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;

  nthw_module_t *mHsh;

  nthw_register_t *mpRcpCtrl;
  nthw_field_t *mpRcpAddr;
  nthw_field_t *mpRcpCnt;
  nthw_register_t *mpRcpData;
  nthw_field_t *mpRcpDataLoadDistType;
  nthw_field_t *mpRcpDataMacPortMask;
  nthw_field_t *mpRcpDataSort;
  nthw_field_t *mpRcpDataQw0Pe;
  nthw_field_t *mpRcpDataQw0Ofs;
  nthw_field_t *mpRcpDataQw4Pe;
  nthw_field_t *mpRcpDataQw4Ofs;
  nthw_field_t *mpRcpDataW8Pe;
  nthw_field_t *mpRcpDataW8Ofs;
  nthw_field_t *mpRcpDataW8Sort;
  nthw_field_t *mpRcpDataW9Pe;
  nthw_field_t *mpRcpDataW9Ofs;
  nthw_field_t *mpRcpDataW9Sort;
  nthw_field_t *mpRcpDataW9P;
  nthw_field_t *mpRcpDataPMask;
  nthw_field_t *mpRcpDataWordMask;
  nthw_field_t *mpRcpDataSeed;
  nthw_field_t *mpRcpDataTnlP;
  nthw_field_t *mpRcpDataHshValid;
  nthw_field_t *mpRcpDataHshType;
  nthw_field_t *mpRcpDataAutoIpv4Mask;
};

#endif /* __FLOW_NTHW_HSH_H__ */
