/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_HST_H__
#define __FLOW_NTHW_HST_H__

#include <stdint.h> /* uint32_t */
#include "nthw_fpga_model.h"


struct HstNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;

  nthw_module_t *mHst;

  nthw_register_t *mpRcpCtrl;
  nthw_field_t *mpRcpAddr;
  nthw_field_t *mpRcpCnt;
  nthw_register_t *mpRcpData;
  nthw_field_t *mpRcpDataStripMode;
  nthw_field_t *mpRcpDataStartDyn;
  nthw_field_t *mpRcpDataStartOfs;
  nthw_field_t *mpRcpDataEndDyn;
  nthw_field_t *mpRcpDataEndOfs;
  nthw_field_t *mpRcpDataModif0Cmd;
  nthw_field_t *mpRcpDataModif0Dyn;
  nthw_field_t *mpRcpDataModif0Ofs;
  nthw_field_t *mpRcpDataModif0Value;
  nthw_field_t *mpRcpDataModif1Cmd;
  nthw_field_t *mpRcpDataModif1Dyn;
  nthw_field_t *mpRcpDataModif1Ofs;
  nthw_field_t *mpRcpDataModif1Value;
  nthw_field_t *mpRcpDataModif2Cmd;
  nthw_field_t *mpRcpDataModif2Dyn;
  nthw_field_t *mpRcpDataModif2Ofs;
  nthw_field_t *mpRcpDataModif2Value;
};

typedef struct HstNthw HstNthw_t;

struct HstNthw *HstNthw_new(void);
void HstNthw_delete(struct HstNthw *p);
int HstNthw_init(struct HstNthw *p, nthw_fpga_t *pFpga, int nInstance);

int HstNthw_setup(struct HstNthw *p, int nIdx, int nIdxCnt);
void hstNthw_setDebugMode(struct HstNthw *p, unsigned int nDebugMode);

#if 0
uint32_t GetVersionMajor(void) {return (mHst.getVersion()).getMajorVersion();}
uint32_t GetVersionMinor(void) {return (mHst.getVersion()).getMinorVersion();}
void SetDebugOutput(int on) {if (on) mHst.setDebugMode(Register::ON_WRITE); else mHst.setDebugMode(Register::NO_DEBUG);}
#endif

/* RCP */
void HstNthw_RcpSelect(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpCnt(const struct HstNthw *p, uint32_t val);

void HstNthw_RcpStripMode(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpStartDyn(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpStartOfs(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpEndDyn(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpEndOfs(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpModif0Cmd(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpModif0Dyn(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpModif0Ofs(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpModif0Value(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpModif1Cmd(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpModif1Dyn(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpModif1Ofs(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpModif1Value(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpModif2Cmd(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpModif2Dyn(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpModif2Ofs(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpModif2Value(const struct HstNthw *p, uint32_t val);
void HstNthw_RcpFlush(const struct HstNthw *p);

#endif /* __FLOW_NTHW_HST_H__ */
