/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#include "ntlog.h"
#include "nthw_drv.h"
#include "nthw_register.h"

#include "flow_nthw_ifr.h"

void ifrNthw_setDebugMode(struct IfrNthw *p, unsigned int nDebugMode)
{
  nthw_module_set_debug_mode(p->mIfr, nDebugMode);
}

struct IfrNthw *IfrNthw_new(void)
{
  struct IfrNthw *p = malloc(sizeof(struct IfrNthw));
  if (p) {
    (void)memset(p, 0, sizeof(*p));
  }
  return p;
}

void IfrNthw_delete(struct IfrNthw *p)
{
  if (p) {
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}


int IfrNthw_init(struct IfrNthw *p, nthw_fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->p_fpga_info->mpAdapterIdStr;
  nthw_module_t* pMod = Fpga_queryModule(pFpga, MOD_IFR, nInstance);

  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: Ifr %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mIfr = Fpga_queryModule(pFpga, MOD_IFR, nInstance);

  p->mpRcpCtrl = nthw_module_get_register(p->mIfr, IFR_RCP_CTRL);
  p->mpRcpAddr = nthw_register_get_field(p->mpRcpCtrl, IFR_RCP_CTRL_ADR);
  p->mpRcpCnt = nthw_register_get_field(p->mpRcpCtrl, IFR_RCP_CTRL_CNT);
  p->mpRcpData = nthw_module_get_register(p->mIfr, IFR_RCP_DATA);
  p->mpRcpDataEn = nthw_register_get_field(p->mpRcpData, IFR_RCP_DATA_EN);
  p->mpRcpDataMtu = nthw_register_get_field(p->mpRcpData, IFR_RCP_DATA_MTU);

  return 0;
}

void IfrNthw_RcpSelect(const struct IfrNthw *p, uint32_t val)
{
  assert(p->mpRcpAddr);
  nthw_field_set_val32(p->mpRcpAddr, val);
}

void IfrNthw_RcpCnt(const struct IfrNthw *p, uint32_t val)
{
  assert(p->mpRcpCnt);
  nthw_field_set_val32(p->mpRcpCnt, val);
}

void IfrNthw_RcpEn(const struct IfrNthw *p, uint32_t val)
{
  assert(p->mpRcpDataEn);
  nthw_field_set_val32(p->mpRcpDataEn, val);
}

void IfrNthw_RcpMtu(const struct IfrNthw *p, uint32_t val)
{
  assert(p->mpRcpDataEn);
  nthw_field_set_val32(p->mpRcpDataMtu, val);
}

void IfrNthw_RcpFlush(const struct IfrNthw *p)
{
  assert(p->mpRcpCtrl);
  assert(p->mpRcpData);
  nthw_register_flush(p->mpRcpCtrl, 1);
  nthw_register_flush(p->mpRcpData, 1);
}

