/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_PDB_H__
#define __FLOW_NTHW_PDB_H__

#include <stdint.h> /* uint32_t */
#include "nthw_fpga_model.h"

struct PdbNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;

  nthw_module_t *mPdb;

  nthw_register_t *mpRcpCtrl;
  nthw_field_t *mpRcpAddr;
  nthw_field_t *mpRcpCnt;
  nthw_register_t *mpRcpData;
  nthw_field_t *mpRcpDataDescriptor;
  nthw_field_t *mpRcpDataDescLen;
  nthw_field_t *mpRcpDataTxPort;
  nthw_field_t *mpRcpDataTxIgnore;
  nthw_field_t *mpRcpDataTxNow;
  nthw_field_t *mpRcpDataCrcOverwrite;
  nthw_field_t *mpRcpDataAlign;
  nthw_field_t *mpRcpDataOfs0Dyn;
  nthw_field_t *mpRcpDataOfs0Rel;
  nthw_field_t *mpRcpDataOfs1Dyn;
  nthw_field_t *mpRcpDataOfs1Rel;
  nthw_field_t *mpRcpDataOfs2Dyn;
  nthw_field_t *mpRcpDataOfs2Rel;
  nthw_field_t *mpRcpDataIpProtTnl;
  nthw_field_t *mpRcpDataPpcHsh;
  nthw_field_t *mpRcpDataDuplicateEn;
  nthw_field_t *mpRcpDataDuplicateBit;
  nthw_field_t *mpRcpDataPcapKeepFcs;

  nthw_register_t *mpConfig;
  nthw_field_t *mpConfigTsFormat;
  nthw_field_t *mpConfigPortOfs;
};

typedef struct PdbNthw PdbNthw_t;
#if 0
typedef struct PdbNthw PdbNthw;
#endif

struct PdbNthw *PdbNthw_new(void);
void PdbNthw_delete(struct PdbNthw *p);
int PdbNthw_init(struct PdbNthw *p, nthw_fpga_t *pFpga, int nInstance);

int PdbNthw_setup(struct PdbNthw *p, int nIdx, int nIdxCnt);
void pdbNthw_setDebugMode(struct PdbNthw *p, unsigned int nDebugMode);

#if 0
uint32_t GetVersionMajor(void) {return (mPdb.getVersion()).getMajorVersion();}
uint32_t GetVersionMinor(void) {return (mPdb.getVersion()).getMinorVersion();}
void SetDebugOutput(int on) {if (on) mPdb.setDebugMode(Register::ON_WRITE); else mPdb.setDebugMode(Register::NO_DEBUG);}
#endif

/* RCP */
void PdbNthw_RcpSelect(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpCnt(const struct PdbNthw *p, uint32_t val);

void PdbNthw_RcpDescriptor(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpDescLen(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpTxPort(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpTxIgnore(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpTxNow(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpCrcOverwrite(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpAlign(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpOfs0Dyn(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpOfs0Rel(const struct PdbNthw *p, int32_t val);
void PdbNthw_RcpOfs1Dyn(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpOfs1Rel(const struct PdbNthw *p, int32_t val);
void PdbNthw_RcpOfs2Dyn(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpOfs2Rel(const struct PdbNthw *p, int32_t val);
void PdbNthw_RcpIpProtTnl(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpPpcHsh(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpDuplicateEn(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpDuplicateBit(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpDataPcapKeepFcs(const struct PdbNthw *p, uint32_t val);
void PdbNthw_RcpFlush(const struct PdbNthw *p);

/* CONFIG */
void PdbNthw_ConfigTsFormat(const struct PdbNthw *p, uint32_t val);
void PdbNthw_ConfigPortOfs(const struct PdbNthw *p, uint32_t val);
void PdbNthw_ConfigPortOfs(const struct PdbNthw *p, uint32_t val);
void PdbNthw_ConfigFlush(const struct PdbNthw *p);

#endif /* __FLOW_NTHW_PDB_H__ */
