/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __FLOW_NTHW_TX_CPY_H__
#define __FLOW_NTHW_TX_CPY_H__

#include <stdint.h>
#include "nthw_fpga_model.h"

struct TxCpyWriters_s {
  nthw_register_t *mpWriterCtrl;
  nthw_field_t *mpWriterCtrlAddr;
  nthw_field_t *mpWriterCtrlCnt;

  nthw_register_t *mpWriterData;
  nthw_field_t *mpWriterDataReaderSelect;
  nthw_field_t *mpWriterDataDyn;
  nthw_field_t *mpWriterDataOfs;
  nthw_field_t *mpWriterDataLen;
  nthw_field_t *mpWriterDataMaskPointer;

  nthw_register_t *mpWriterMaskCtrl;
  nthw_field_t *mpWriterMaskCtrlAddr;
  nthw_field_t *mpWriterMaskCtrlCnt;

  nthw_register_t *mpWriterMaskData;
  nthw_field_t *mpWriterMaskDataByteMask;
};

struct TxCpyNthw {
  uint8_t mPhysicalAdapterNo;
  nthw_fpga_t *mpFpga;

  nthw_module_t *mTxCpy;

  unsigned int mWritersCnt;
  struct TxCpyWriters_s *mWriters;
};

struct TxCpyNthw *TxCpyNthw_new(void);
void TxCpyNthw_delete(struct TxCpyNthw *p);
int TxCpyNthw_init(struct TxCpyNthw *p, nthw_fpga_t *pFpga, int nInstance);

int TxCpyNthw_setup(struct TxCpyNthw *p, int nIdx, int nIdxCnt);
void txCpyNthw_setDebugMode(struct TxCpyNthw *p, unsigned int nDebugMode);

#if 0
  uint32_t GetVersionMajor(void) {return (mTxCpy.getVersion()).getMajorVersion();}
  uint32_t GetVersionMinor(void) {return (mTxCpy.getVersion()).getMinorVersion();}
  void SetDebugOutput(int on) {if (on) mTxCpy.setDebugMode(Register::ON_WRITE); else mTxCpy.setDebugMode(Register::NO_DEBUG);}
#endif

void TxCpyNthw_WriterSelect(const struct TxCpyNthw *p, unsigned int index, uint32_t val);
void TxCpyNthw_WriterCnt(const struct TxCpyNthw *p, unsigned int index, uint32_t val);
void TxCpyNthw_WriterReaderSelect(const struct TxCpyNthw *p, unsigned int index, uint32_t val);
void TxCpyNthw_WriterDyn(const struct TxCpyNthw *p, unsigned int index, uint32_t val);
void TxCpyNthw_WriterOfs(const struct TxCpyNthw *p, unsigned int index, uint32_t val);
void TxCpyNthw_WriterLen(const struct TxCpyNthw *p, unsigned int index, uint32_t val);
void TxCpyNthw_WriterMaskPointer(const struct TxCpyNthw *p, unsigned int index, uint32_t val);
void TxCpyNthw_WriterFlush(const struct TxCpyNthw *p, unsigned int index);

void TxCpyNthw_WriterMaskSelect(const struct TxCpyNthw *p, unsigned int index, uint32_t val);
void TxCpyNthw_WriterMaskCnt(const struct TxCpyNthw *p, unsigned int index, uint32_t val);
void TxCpyNthw_WriterMask(const struct TxCpyNthw *p, unsigned int index, uint32_t val);
void TxCpyNthw_WriterMaskFlush(const struct TxCpyNthw *p, unsigned int index);

#endif /* __FLOW_NTHW_TX_CPY_H__ */
