/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#include "ntlog.h"
#include "nthw_drv.h"
#include "nthw_register.h"

#include "flow_nthw_tx_ins.h"

#include <stdlib.h>
#include <string.h>

void txInsNthw_setDebugMode(struct TxInsNthw *p, unsigned int nDebugMode)
{
  nthw_module_set_debug_mode(p->mTxIns, nDebugMode);
}

struct TxInsNthw *TxInsNthw_new(void)
{
  struct TxInsNthw *p = malloc(sizeof(struct TxInsNthw));
  if (p) {
    (void)memset(p, 0, sizeof(*p));
  }
  return p;
}

void TxInsNthw_delete(struct TxInsNthw *p)
{
  if (p) {
    (void)memset(p, 0, sizeof(*p));
    free(p);
  }
}

int TxInsNthw_init(struct TxInsNthw *p, nthw_fpga_t *pFpga, int nInstance)
{
  const char *const pAdapterIdStr = pFpga->p_fpga_info->mpAdapterIdStr;
  nthw_module_t* pMod = Fpga_queryModule(pFpga, MOD_TX_INS, nInstance);

  assert(nInstance >= 0 && nInstance < 256);

  if (p == NULL) {
    return pMod == NULL ? -1 : 0;
  }

  if (pMod == NULL) {
    NT_LOG(ERR, NTHW, "%s: TxIns %d: no such instance\n", pAdapterIdStr, nInstance);
    return -1;
  }

  p->mpFpga = pFpga;
  p->mPhysicalAdapterNo = (uint8_t)nInstance;
  p->mTxIns = Fpga_queryModule(pFpga, MOD_TX_INS, nInstance);

  p->mpRcpCtrl = nthw_module_get_register(p->mTxIns, INS_RCP_CTRL);
  p->mpRcpAddr = nthw_register_get_field(p->mpRcpCtrl, INS_RCP_CTRL_ADR);
  p->mpRcpCnt = nthw_register_get_field(p->mpRcpCtrl, INS_RCP_CTRL_CNT);
  p->mpRcpData = nthw_module_get_register(p->mTxIns, INS_RCP_DATA);
  p->mpRcpDataDyn = nthw_register_get_field(p->mpRcpData, INS_RCP_DATA_DYN);
  p->mpRcpDataOfs = nthw_register_get_field(p->mpRcpData, INS_RCP_DATA_OFS);
  p->mpRcpDataLen = nthw_register_get_field(p->mpRcpData, INS_RCP_DATA_LEN);

  return 0;
}

void TxInsNthw_RcpSelect(const struct TxInsNthw *p, uint32_t val)
{
  nthw_field_set_val32(p->mpRcpAddr, val);
}

void TxInsNthw_RcpCnt(const struct TxInsNthw *p, uint32_t val)
{
  nthw_field_set_val32(p->mpRcpCnt, val);
}

void TxInsNthw_RcpDyn(const struct TxInsNthw *p, uint32_t val)
{
  nthw_field_set_val32(p->mpRcpDataDyn, val);
}

void TxInsNthw_RcpOfs(const struct TxInsNthw *p, uint32_t val)
{
  nthw_field_set_val32(p->mpRcpDataOfs, val);
}

void TxInsNthw_RcpLen(const struct TxInsNthw *p, uint32_t val)
{
  nthw_field_set_val32(p->mpRcpDataLen, val);
}

void TxInsNthw_RcpFlush(const struct TxInsNthw *p)
{
  nthw_register_flush(p->mpRcpCtrl, 1);
  nthw_register_flush(p->mpRcpData, 1);
}
