/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef __NTHW_DRV_H__
#define __NTHW_DRV_H__

#include "nthw_profile.h"

typedef enum NtMetaPortType_e {
  PORT_TYPE_PHYSICAL,
  PORT_TYPE_VIRTUAL,
  PORT_TYPE_OVERRIDE,
} NtMetaPortType_t;

#include "nthw_helper.h"
#include "nthw_utils.h"
#include "nthw_platform_drv.h"
#include "nthw_fpga_model.h"
#include "nthw_mcu.h"
#include "nthw_stat.h"
#include "nthw_dbs.h"
#include "nthw_epp.h"
#include "nthw_core.h"


typedef struct McuInfo_s
{
#if 0
  /* platform, memsize, version, built_for_fpga_version ??? */
#endif
  bool mbHasMcu;
  int mnMcuType;
  int mnMcuDramSize;
} McuInfo_t;

typedef struct NthwHwInfo_s
{
  /* From FW */
  int hw_id;
  int hw_id_emulated;
  char hw_plat_id_str[32];

  struct VpdInfo_s {
    int mnMacAddrCount;
    uint64_t mnMacAddrValue;
    uint8_t maMacAddrOctets[6];
  } vpd_info;
} NthwHwInfo_t;

typedef struct FpgaInfo_s
{
  uint64_t nFpgaIdent;

  int nFpgaTypeId;
  int nFpgaProdId;
  int nFpgaVerId;
  int nFpgaRevId;

  int nFpgaBuildTime;

  int nFpgaDebugMode;

  int nNims;
  int nPhyPorts;
  int nPhyQuads;
  int nRxPorts;
  int nTxPorts;

  enum FpgaInfoProfile profile;

  struct nthw_fpga_s* mpFpga;

  struct NthwRac* mpNthwRac;
  struct NthwHif* mpNthwHif;
  struct NthwPcie3* mpNthwPcie3;
  struct NthwTsm* mpNthwTsm;

  NthwDbs_t *mpNthwDbs;
  NthwEpp_t *mpNthwEpp;

  uint8_t* bar0_addr; /* Needed for register read/write */
  size_t   bar0_size;

  int adapterNo;      /* Needed for nthw_rac DMA array indexing */
  uint32_t pciident;  /* Needed for nthw_rac DMA memzone_reserve */
  int      numa_node; /* Needed for nthw_rac DMA memzone_reserve */

  char* mpAdapterIdStr;  /* Pointer to string literal used in nthw log messages */

  struct McuInfo_s mcu_info;

  struct NthwHwInfo_s nthw_hw_info;

  NthwAdapterId_t nNthwAdapterId;

} FpgaInfo_t;

#define DUMP_FUNC printf("FUNC %s\n", __func__)

#endif /* __NTHW_DRV_H__ */
