/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#ifndef NTHW_EPP_HPP_
#define NTHW_EPP_HPP_

/* VXLAN adds extra 50 bytes */
#define VXLANDATASIZEADJUST 50
#define VXLANDATASIZEADJUSTIPV6 70
#define MTUINITVAL 1500
#define NRECIPE 3

/* List of size adjust values to put in the recipe memory data register at startup */
static const int rcpDataSizeAdjustTxp[NRECIPE] = {0, VXLANDATASIZEADJUST, VXLANDATASIZEADJUSTIPV6};
static const int rcpDataSizeAdjustVport[NRECIPE] = {0, VXLANDATASIZEADJUST, VXLANDATASIZEADJUSTIPV6};

struct NthwEpp_s
{
  nthw_fpga_t* mpFpga;
  nthw_module_t* mpModEpp;
  int mnInstance;
  int mnEppCategories;

  nthw_register_t* mpRegReciepeMemoryControl;
  nthw_field_t*    mpFldReciepeMemoryControlAdr;
  nthw_field_t*    mpFldReciepeMemoryControlCnt;

  nthw_register_t* mpRegReciepeMemoryData;
  nthw_field_t*    mpFldReciepeMemoryDataTxMtuEppEnable;
  nthw_field_t*    mpFldReciepeMemoryDataQueueMtuEppEnable;
  nthw_field_t*    mpFldReciepeMemoryDataSizeAdjustTxPort;
  nthw_field_t*    mpFldReciepeMemoryDataSizeAdjustVirtualPort;
  nthw_field_t*    mpFldReciepeMemoryDataFixed18bL2Mtu;
  nthw_field_t*    mpFldReciepeMemoryDataTxpQosEppEnable;
  nthw_field_t*    mpFldReciepeMemoryDataQueueQosEppEnable;

  nthw_register_t* mpRegTxpPortMtuControl;
  nthw_field_t*    mpFldTxpPortMtuControlAdr;
  nthw_field_t*    mpFldTxpPortMtuControlCnt;

  nthw_register_t* mpRegTxpPortMtuData;
  nthw_field_t*    mpFldTxpPortMtuDataMaxMtu;

  nthw_register_t* mpRegQueueMtuControl;
  nthw_field_t*    mpFldQueueMtuControlAdr;
  nthw_field_t*    mpFldQueueMtuControlCnt;

  nthw_register_t* mpRegQueueMtuData;
  nthw_field_t*    mpFldQueueMtuDataMaxMtu;

  nthw_register_t* mpRegTxpQosControl;
  nthw_field_t*    mpFldTxpQosControlAdr;
  nthw_field_t*    mpFldTxpQosControlCnt;

  nthw_register_t* mpRegTxpQosData;
  nthw_field_t*    mpFldTxpQosDataEnable;
  nthw_field_t*    mpFldTxpQosDataInformationRate;
  nthw_field_t*    mpFldTxpQosDataInformationRateFractional;
  nthw_field_t*    mpFldTxpQosDataBurstSize;

  nthw_register_t* mpRegVportQosControl;
  nthw_field_t*    mpFldVportQosControlAdr;
  nthw_field_t*    mpFldVportQosControlCnt;

  nthw_register_t* mpRegVportQosData;
  nthw_field_t*    mpFldVportQosDataEnable;
  nthw_field_t*    mpFldVportQosDataInformationRate;
  nthw_field_t*    mpFldVportQosDataInformationRateFractional;
  nthw_field_t*    mpFldVportQosDataBurstSize;

  nthw_register_t* mpRegQueueVportControl;
  nthw_field_t*    mpFldQueueVportControlAdr;
  nthw_field_t*    mpFldQueueVportControlCnt;

  nthw_register_t* mpRegQueueVportData;
  nthw_field_t*    mpFldQueueVportDataVport;
};

typedef struct NthwEpp_s NthwEpp_t;

NthwEpp_t* NthwEpp_new(void);
void NthwEpp_delete(NthwEpp_t* p);

int NthwEpp_present(nthw_fpga_t* pFpga, int nInstance);
int NthwEpp_init(NthwEpp_t *p, nthw_fpga_t* pFpga, int nInstance);
int NthwEpp_setup(NthwEpp_t *p);
int NthwEpp_setMtu(NthwEpp_t *p, uint32_t port, uint32_t maxMtu, NtMetaPortType_t portType);
int NthwEpp_setTxpQos(NthwEpp_t *p, uint32_t port, uint32_t informationRate, uint32_t informationRateFractional, uint32_t burstSize);
int NthwEpp_setVportQos(NthwEpp_t *p, uint32_t port, uint32_t informationRate, uint32_t informationRateFractional, uint32_t burstSize);
int NthwEpp_setQueueToVport(NthwEpp_t *p, uint32_t qid, uint32_t vport);

#endif /* NTHW_EPP_HPP_ */
