/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#include "avr_sensors.h"
#include "ntlog.h"

#define MAX_ADAPTERS  2

uint8_t sFpgaIndexes[MAX_ADAPTERS] = {0}; /* _NTSD_MAX_NUM_ADAPTERS_ */
static uint8_t getFpgaIdx(unsigned int adapterNo);

/*
 * This function setups monitoring of AVR sensors
 */
static uint8_t _avr_sensor_init(Nthw_Spi_V3_t *s_spi, uint8_t mAdapterNo, const char* pName,
                                enum sensor_mon_device avrDev, uint8_t avrDevReg,
                                enum sensor_mon_endian end, enum sensor_mon_sign si,
                                uint16_t mask)
{
    uint8_t fpgaIdx = getFpgaIdx(mAdapterNo);
    struct sensor_mon_setup16 AvrSensorSetup;

    /* Setup monitoring in AVR placing results in FPGA */
    AvrSensorSetup.setup_cnt = 1;
    AvrSensorSetup.setup_data[0].fpga_idx = fpgaIdx;
    AvrSensorSetup.setup_data[0].device = avrDev;
    AvrSensorSetup.setup_data[0].device_register = avrDevReg;
    AvrSensorSetup.setup_data[0].format = (uint16_t)(end | si << 2);

    AvrSensorSetup.setup_data[0].mask = mask;
    AvrSensorSetup.setup_data[0].pos  = 0; /* So far for all sensors in table */

    /* At first it is the task of ntservice to test limitLow and limitHigh on all */
    /* board sensors. Later the test is going to be carried out by the AVR */
    if (si == SENSOR_MON_SIGNED)
    {
        AvrSensorSetup.setup_data[0].int16.limitLow = SENSOR_MON_INT16_NAN;
        AvrSensorSetup.setup_data[0].int16.limitHigh = SENSOR_MON_INT16_NAN;
    }
    else
    {
        AvrSensorSetup.setup_data[0].uint16.limitLow = SENSOR_MON_UINT16_NAN;
        AvrSensorSetup.setup_data[0].uint16.limitHigh = SENSOR_MON_UINT16_NAN;
    }

    int result = NtAvrSensorMonSetup(&AvrSensorSetup, s_spi);
    if (result) {
        NT_LOG(ERR, ETHDEV, "%s: sensor initialization error\n", pName);
    }

    return fpgaIdx;
}

static void avr_read(struct nt_sensor_group* sg, NthwSpis_t* t_spi) {
    uint32_t pSensorResult;

    if(sg == NULL || sg->sensor == NULL){return;}

    sensor_read(t_spi, sg->sensor->fpgaIdx, &pSensorResult);
    update_sensor_value(sg->sensor, sg->conv_func(pSensorResult));
}

struct nt_sensor_group* avr_sensor_init(Nthw_Spi_V3_t *s_spi, uint8_t mAdapterNo, const char* pName,
                                        enum NtSensorSource_e ssrc, enum NtSensorType_e type,
                                        unsigned int index,
                                        enum sensor_mon_device avrDev, uint8_t avrDevReg,
                                        enum sensor_mon_endian end, enum sensor_mon_sign si,
                                        int (*conv_func)(uint32_t),
                                        uint16_t mask)
{
    struct nt_sensor_group* sg = malloc(sizeof(struct nt_sensor_group));
    if (sg == NULL) {
        NT_LOG(ERR, ETHDEV, "%s: sensor group is NULL", __func__);
        return NULL;
    }
    init_sensor_group(sg);
    sg->sensor = allocate_sensor(mAdapterNo, pName, ssrc, type, index, NT_SENSOR_DISABLE_ALARM, si);
    sg->sensor->fpgaIdx = _avr_sensor_init(s_spi, mAdapterNo, pName, avrDev, avrDevReg, end, si, mask);
    sg->read = &avr_read;
    sg->conv_func = conv_func;
    sg->monitor = NULL;
    sg->next = NULL;
    return sg;
}

static uint8_t getFpgaIdx(unsigned int adapterNo)
{
  uint8_t tmp = sFpgaIndexes[adapterNo];
  sFpgaIndexes[adapterNo] = (uint8_t)(tmp + 1);

  return tmp;
}
