/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright(c) 2023 Napatech A/S
 */

#include "tempmon.h"
#include "ntlog.h"
#include "nthw_register.h"

struct nt_fpga_sensor_monitor* Tempmon_new(void) {
    struct nt_fpga_sensor_monitor* temp = malloc(sizeof(struct nt_fpga_sensor_monitor));
    if (temp == NULL) {
        NT_LOG(ERR, ETHDEV, "%s: monitor is NULL\n", __func__);
    }
    return temp;
}

void Tempmon_init(struct nt_fpga_sensor_monitor *t, nthw_fpga_t* pFpga) {
    if (t == NULL || pFpga == NULL) {
        NT_LOG(ERR, ETHDEV, "%s: bad argument(s)\n", __func__);
        return;
    }
    /* fetch initialized module */
    t->FPGA = pFpga;
    t->mod = nthw_get_module(t->FPGA, MOD_TEMPMON, 0);
    if(t->mod == NULL) {
        NT_LOG(ERR, ETHDEV, "module is NULL\n");
    }
    /* fetch register */
    t->reg = nthw_module_get_register(t->mod, TEMPMON_STAT);
    if(t->reg == NULL) {
        NT_LOG(ERR, ETHDEV, "register is NULL\n");
    }
    /* fetch fields */
    t->fields = malloc(sizeof(nthw_field_t*));
    if(t->fields == NULL){
        NT_LOG(ERR, ETHDEV, "%s: field is NULL", __func__);
        return;
    }
    t->fields[0] = nthw_register_get_field(t->reg, TEMPMON_STAT_TEMP);
    if(t->fields[0] == NULL) {
        NT_LOG(ERR, ETHDEV, "field is NULL\n");
    }
}
