/*-
 *   BSD LICENSE
 *
 *   Copyright(c) 2010-2015 Intel Corporation. All rights reserved.
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <unistd.h>
#include <stdint.h>
#include <stdbool.h>
#include <inttypes.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <linux/limits.h>
#include <getopt.h>
#include <termios.h>
#include <time.h>

#include <rte_eal.h>
#include <rte_ethdev.h>
#include <rte_cycles.h>
#include <rte_lcore.h>
#include <rte_mbuf.h>
#include <rte_flow.h>
#include <rte_flow_driver.h>
#include <rte_net.h>
#include <rte_mtr.h>
#include <rte_mtr_driver.h>

#include <rte_malloc.h>

#include "filter.h"

extern int meter_mtr_id[NO_PORTS];


struct decap_encap_s decap0Buffer = {
  .eth = {
    .dst_addr.addr_bytes = "\xff\xff\xff\xff\xff\xff",
    .src_addr.addr_bytes = "\xff\xff\xff\xff\xff\xff",
    .ether_type = RTE_BE16(RTE_ETHER_TYPE_VLAN)
  },
  .vlan = {
    .eth_proto = RTE_BE16(RTE_ETHER_TYPE_IPV4),
    .vlan_tci = RTE_BE16(44)
  },
  .ipv4 = {
    .version_ihl = 0x45,
    .src_addr = 0xFFFFFFFF,
    .dst_addr = 0xFFFFFFFF,
    .type_of_service = 0,
    .time_to_live = 0,
    .next_proto_id = IPPROTO_UDP
  },
  .udp = {
    .src_port = RTE_BE16(0),
    .dst_port = RTE_BE16(2152)
  },
  .gtp = {
    .ver = 1,
    .pt = 1,
    .e = 1,
    .msg_type = 255,
    .teid = RTE_BE32(0x00000000)
  },
  .ext_word = {
    .next_ext = 0x85
  },
  .gtp_psc = {
    .ext_hdr_len = 1,
    .type = 1,
    .spare = 0,
    .ppp = 0,
    .rqi = 1,
    .qfi = 6,
    .next_ext = 0
  }
};

struct decap_encap_s encap1Buffer = {
  .eth = {
    .dst_addr.addr_bytes = "\x22\x33\x44\x55\x66\x77",
    .src_addr.addr_bytes = "\xff\xff\xff\xff\xff\xff",
    .ether_type = RTE_BE16(RTE_ETHER_TYPE_VLAN)
  },
  .vlan = {
    .eth_proto = RTE_BE16(RTE_ETHER_TYPE_IPV4),
    .vlan_tci = 0
  },
  .ipv4 = {
    .version_ihl = 0x45,
    .src_addr = 0x0A0A0A05,
    .dst_addr = 0x0B0B0B07,
    .type_of_service = 10,
    .time_to_live = 0x40,
    .next_proto_id = IPPROTO_UDP
  },
  .udp = {
    .src_port = RTE_BE16(1000),
    .dst_port = RTE_BE16(2152)
  },
  .gtp = {
    .ver = 1,
    .pt = 1,
    .e = 1,
    .msg_type = 255,
    .teid = RTE_BE32(0x00000001)
  },
  .ext_word = {
    .next_ext = 0x85
  },
  .gtp_psc = {
    .ext_hdr_len = 1,
    .type = 1,
    .spare = 0,
    .ppp = 0,
    .rqi = 1,
    .qfi = 6,
    .next_ext = 0
  }
};

struct encap_decap_s encap0Buffer = {
  .eth = {
    .dst_addr.addr_bytes = "\x26\x26\x26\x26\x26\x26",
    .src_addr.addr_bytes = "\x2A\x2A\x2A\x2A\x2A\x2A",
    .ether_type = RTE_BE16(RTE_ETHER_TYPE_VLAN)
  },
  .vlan = {
    .eth_proto = RTE_BE16(RTE_ETHER_TYPE_IPV4),
    .vlan_tci = RTE_BE16(45)
  },
};

struct encap_decap_s decap1Buffer = {
  .eth = {
    .dst_addr.addr_bytes = "\xff\xff\xff\xff\xff\xff",
    .src_addr.addr_bytes = "\xff\xff\xff\xff\xff\xff",
    .ether_type = RTE_BE16(RTE_ETHER_TYPE_VLAN)
  },
  .vlan = {
    .eth_proto = RTE_BE16(RTE_ETHER_TYPE_IPV4),
    .vlan_tci = RTE_BE16(0)
  },
};

extern bool use_meter;
extern bool use_decap;
extern bool debug_print_out;
extern uint32_t ingress_port;
extern uint32_t eegress_port;

extern pthread_mutex_t mutex;

void set_eth_mac_address(uint16_t port, struct rte_ether_addr *mac_addr)
{
  if (port == eegress_port) {
    memcpy(encap0Buffer.eth.src_addr.addr_bytes, mac_addr->addr_bytes, 6);
  }
  else {
    memcpy(encap1Buffer.eth.src_addr.addr_bytes, mac_addr->addr_bytes, 6);
  }
}

void set_vlan(uint16_t port, uint16_t tci)
{
  if (port == eegress_port) {
    encap0Buffer.vlan.vlan_tci = RTE_BE16(tci);
  }
  else {
    encap1Buffer.vlan.vlan_tci = RTE_BE16(tci);
  }
}

#ifdef DUMP_PACKETS
static void dump_ipv4_header(struct rte_ipv4_hdr *ip)
{
  printf("ipv4 header\n");
  printf("version_ihl:     %02X\n", ip->version_ihl);
  printf("type_of_service: %02X\n", ip->type_of_service);
  printf("total_length:    %04X\n", RTE_BE16(ip->total_length));
  printf("packet_id:       %04X\n", RTE_BE16(ip->packet_id));
  printf("fragment_offset: %04X\n", RTE_BE16(ip->fragment_offset));
  printf("time_to_live:    %02X\n", ip->time_to_live);
  printf("next_proto_id:   %02X\n", ip->next_proto_id);
  printf("hdr_checksum:    %04X\n", RTE_BE16(ip->hdr_checksum));
  printf("src_addr:        %d.%d.%d.%d\n", IPV4_ADDRESS(ip->src_addr));
  printf("dst_addr:        %d.%d.%d.%d\n", IPV4_ADDRESS(ip->dst_addr));
}

static void dump_udp_header(struct rte_udp_hdr *hdr) {
  printf("UDP header\n");
  printf("src_port:     %04X\n", RTE_BE16(hdr->src_port));
  printf("dst_port:     %04X\n", RTE_BE16(hdr->dst_port));
  printf("dgram_len:    %04X\n", RTE_BE16(hdr->dgram_len));
  printf("dgram_cksum:  %04X\n", RTE_BE16(hdr->dgram_cksum));
}

static void dump_gtp_header(struct rte_gtp_hdr *gtp)
{
  uint8_t *ptr;
  printf("GTP header\n");
  printf("pn:       %X\n",gtp->pn);
  printf("s:        %X\n",gtp->s);
  printf("e:        %X\n",gtp->e);
  printf("res1:     %X\n",gtp->res1);
  printf("pt:       %X\n",gtp->pt);
  printf("ver:      %X\n",gtp->ver);
  printf("msg_type: %02X\n",gtp->msg_type);
  printf("plen:     %04X\n",RTE_BE16(gtp->plen));
  printf("teid:     %08X\n",RTE_BE32(gtp->teid));
  if (gtp->e) {
    ptr = (uint8_t*)((uint8_t*)gtp + sizeof(struct rte_gtp_hdr));
    uint8_t next_ext = ((struct rte_gtp_hdr_ext_word*)(ptr) )->next_ext;
    printf("next_ext: %02X\n", next_ext);
    ptr += sizeof(struct rte_gtp_hdr_ext_word);
    while (next_ext) {
      size_t ext_len = (*ptr) * 4;
      printf("ext_len:  %lX\n", ext_len);
      next_ext = *((uint8_t *)(ptr + ext_len - 1));
      printf("next_ext: %02X\n", ((struct rte_gtp_hdr_ext_word*)(ptr) )->next_ext);
      ptr += ext_len;
    }
  }
}

static void dump_eth_header(struct rte_ether_hdr *eth)
{
  int i;
  printf("ETH header\n");
  printf("dst_addr:   ");
  for (i = 0; i < RTE_ETHER_ADDR_LEN; i++) printf("%02X ", eth->dst_addr.addr_bytes[i]);
  printf("\n");
  printf("src_addr:   ");
  for (i = 0; i < RTE_ETHER_ADDR_LEN; i++) printf("%02X ", eth->src_addr.addr_bytes[i]);
  printf("\n");
  printf("ether_type: %04X\n", eth->ether_type);
}

static void dump_vlan_header(struct rte_vlan_hdr *vlan)
{
  printf("VLAN header\n");
  printf("vlan_tci:  %04X\n", vlan->vlan_tci);
  printf("eth_proto: %04X\n", vlan->eth_proto);
}
#endif

static int packet_decoder(const uint8_t *data, struct packet_fields *fields)
{
  uint16_t ether_type;
  uint8_t protocol;
  uint8_t innerL3;

  // L2 - Ethernet frame
  ether_type = RTE_BE16(((const struct rte_ether_hdr *)data)->ether_type);
  data +=  sizeof(struct rte_ether_hdr);;

  while (ether_type == RTE_ETHER_TYPE_VLAN ||
         ether_type == RTE_ETHER_TYPE_QINQ) {
    if (ether_type == RTE_ETHER_TYPE_VLAN) {
      fields->vlan_id = RTE_BE16(((const struct rte_vlan_hdr*)data)->vlan_tci);
      fields->packet_type |= RTE_PTYPE_L2_ETHER_VLAN;
    }
    ether_type = RTE_BE16(((const struct rte_vlan_hdr*)data)->eth_proto);
    data += sizeof(struct rte_vlan_hdr);
  }

  // L3
  if (ether_type == RTE_ETHER_TYPE_IPV4 && ((const struct rte_ipv4_hdr*)data)->version == 4) {
    protocol = ((const struct rte_ipv4_hdr*)data)->next_proto_id;
    fields->match_ipv4_src = ((const struct rte_ipv4_hdr*)data)->src_addr;
    fields->match_ipv4_dst = ((const struct rte_ipv4_hdr*)data)->dst_addr;
    data += sizeof(struct rte_ipv4_hdr);
    fields->packet_type |= RTE_PTYPE_L3_IPV4;
  }
  else if (ether_type == RTE_ETHER_TYPE_IPV6) {
    protocol =  ((const struct rte_ipv6_hdr *)data)->proto;
    memcpy(fields->match_ipv6_src, ((const struct rte_ipv6_hdr *)data)->src_addr, 16);
    memcpy(fields->match_ipv6_dst, ((const struct rte_ipv6_hdr *)data)->dst_addr, 16);
    data += sizeof(struct rte_ipv6_hdr);
    fields->packet_type |= RTE_PTYPE_L3_IPV6;
  } else {
    return 1;  // Unsupported packet
  }

  // L4
  if (protocol == 0x11) {  // UDP
    fields->match_udp_src = ((const struct rte_udp_hdr *)data)->src_port;
    fields->match_udp_dst = ((const struct rte_udp_hdr *)data)->dst_port;
    fields->packet_type |= RTE_PTYPE_L4_UDP;
    data += sizeof(struct rte_udp_hdr);
    if (RTE_BE16(fields->match_udp_dst) != 2152) { // UDP PORT 2152 is GTP TRAFFIC
      return 3;
    }
  }
  else if (protocol == 0x06) { // TCP
    fields->packet_type |= RTE_PTYPE_L4_TCP;
    fields->match_tcp_src = ((const struct rte_tcp_hdr *)data)->src_port;
    fields->match_tcp_dst = ((const struct rte_tcp_hdr *)data)->dst_port;
    return 0;
  }
  else if (protocol == 0x84) { // SCTP
    fields->packet_type |= RTE_PTYPE_L4_SCTP;
    fields->match_sctp_src = ((const struct rte_sctp_hdr *)data)->src_port;
    fields->match_sctp_dst = ((const struct rte_sctp_hdr *)data)->dst_port;
    return 0;
  }
  else   {
    return 2;  // Unsupported packet
  }

  fields->packet_type |= RTE_PTYPE_TUNNEL_GTPU;

  fields->encap_gtp_teid = ((const struct rte_gtp_hdr *)data)->teid;
  uint8_t extension_present_bit = ((const struct rte_gtp_hdr *)data)->e;
  data += sizeof(struct rte_gtp_hdr);
  if (extension_present_bit) {
    uint8_t next_ext = ((const struct rte_gtp_hdr_ext_word*)data)->next_ext;
    data += sizeof(struct rte_gtp_hdr_ext_word);
    while (next_ext) {
      size_t ext_len = *data * 4;
      next_ext = data[ext_len - 1];
      data += ext_len;
    }
  }

  innerL3 = ((*data & 0xf0) >> 4);
  if (innerL3 == 4) { // encapsulated IPv4 packet
    // L3 inner
    protocol = ((const struct rte_ipv4_hdr*)data)->next_proto_id;
    fields->match_ipv4_src_inner = ((const struct rte_ipv4_hdr*)data)->src_addr;
    fields->match_ipv4_dst_inner = ((const struct rte_ipv4_hdr*)data)->dst_addr;
    data += sizeof(struct rte_ipv4_hdr);
    fields->packet_type |= RTE_PTYPE_INNER_L3_IPV4;
  }
  else if (innerL3 == 6) {// encapsulated IPv6 packet
    // L3 inner
    protocol =  ((const struct rte_ipv6_hdr *)data)->proto;
    memcpy(fields->match_ipv6_src_inner, ((const struct rte_ipv6_hdr *)data)->src_addr, 16);
    memcpy(fields->match_ipv6_dst_inner, ((const struct rte_ipv6_hdr *)data)->dst_addr, 16);
    data += sizeof(struct rte_ipv6_hdr);
    fields->packet_type |= RTE_PTYPE_INNER_L3_IPV6;
  }
  else {
    return 4;
  }

  // L4 inner
  if (protocol == 0x11) {  // UDP
    fields->match_udp_src_inner = ((const struct rte_udp_hdr *)data)->src_port;
    fields->match_udp_dst_inner = ((const struct rte_udp_hdr *)data)->dst_port;
    fields->packet_type |= RTE_PTYPE_INNER_L4_UDP;
  }
  else if (protocol == 0x06) { // TCP
    fields->packet_type |= RTE_PTYPE_INNER_L4_TCP;
    fields->match_tcp_src_inner = ((const struct rte_tcp_hdr *)data)->src_port;
    fields->match_tcp_dst_inner = ((const struct rte_tcp_hdr *)data)->dst_port;
  }
  else if (protocol == 0x84) { // SCTP
    fields->packet_type |= RTE_PTYPE_INNER_L4_SCTP;
    fields->match_sctp_src_inner = ((const struct rte_sctp_hdr *)data)->src_port;
    fields->match_sctp_dst_inner = ((const struct rte_sctp_hdr *)data)->dst_port;
  }
  else {
    return 5;  // Unsupported packet
  }
  return 0;
}

static void dump_decoded_packet(struct packet_fields *fields, uint8_t port) {
  pthread_mutex_lock(&mutex);
  printf("P%u - ", port);
  if (fields->packet_type & RTE_PTYPE_L2_ETHER_VLAN) {
    printf("VLAN: %u - ", fields->vlan_id);
  }
  if (fields->packet_type & RTE_PTYPE_L3_IPV4) {
    printf("IPv4 SRC: %d.%d.%d.%d DST: %d.%d.%d.%d - ", IPV4_ADDRESS(fields->match_ipv4_src), IPV4_ADDRESS(fields->match_ipv4_dst));
  } else if (fields->packet_type & RTE_PTYPE_L3_IPV6) {
    printf("IPv6 (%u) SRC: %02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x DST: %02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x - ",
           port, IPV6_ADDRESS(fields->match_ipv6_src), IPV6_ADDRESS(fields->match_ipv6_dst));
  }
  if (fields->packet_type & RTE_PTYPE_L4_UDP) {
    printf("UDP SRC: %d DST: %d\n", (int)RTE_BE16(fields->match_udp_src), (int)RTE_BE16(fields->match_udp_dst));
  }
  else if (fields->packet_type & RTE_PTYPE_L4_TCP) {
    printf("TCP SRC: %d DST: %d\n", (int)RTE_BE16(fields->match_tcp_src), (int)RTE_BE16(fields->match_tcp_dst));
    pthread_mutex_unlock(&mutex);
    return;
  }
  else if (fields->packet_type & RTE_PTYPE_L4_SCTP) {
    printf("SCTP SRC: %d DST: %d\n", (int)RTE_BE16(fields->match_sctp_src), (int)RTE_BE16(fields->match_sctp_dst));
    pthread_mutex_unlock(&mutex);
    return;
  }
  printf("     GTPU TEID 0x%08x - ", RTE_BE32(fields->encap_gtp_teid));
  if (fields->packet_type & RTE_PTYPE_INNER_L3_IPV4) {
    printf("Inner IPv4 SRC: %d.%d.%d.%d DST: %d.%d.%d.%d - ", IPV4_ADDRESS(fields->match_ipv4_src_inner), IPV4_ADDRESS(fields->match_ipv4_dst_inner));
  } else if (fields->packet_type & RTE_PTYPE_INNER_L3_IPV6) {
    printf("Inner IPv6 SRC: %02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x DST: %02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x - ",
           IPV6_ADDRESS(fields->match_ipv6_src_inner), IPV6_ADDRESS(fields->match_ipv6_dst_inner));
  }
  if (fields->packet_type & RTE_PTYPE_INNER_L4_UDP) {
    printf("Inner UDP SRC: %d DST: %d\n", (int)RTE_BE16(fields->match_udp_src_inner), (int)RTE_BE16(fields->match_udp_dst_inner));
  }
  else if (fields->packet_type & RTE_PTYPE_INNER_L4_TCP) {
    printf("Inner TCP SRC: %d DST: %d\n", (int)RTE_BE16(fields->match_tcp_src_inner), (int)RTE_BE16(fields->match_tcp_dst_inner));
  }
  else if (fields->packet_type & RTE_PTYPE_INNER_L4_SCTP) {
    printf("Inner SCTP SRC: %d DST: %d\n", (int)RTE_BE16(fields->match_sctp_src_inner), (int)RTE_BE16(fields->match_sctp_dst_inner));
  }
  pthread_mutex_unlock(&mutex);
}

//#define _PERF_ 1
#ifdef _PERF_
uint64_t tsc1 = 0, tsc2 = 0;
uint64_t counter = 0;
uint64_t diff = 0;
#endif

int offload_filter_setup(struct rte_mbuf *mb,
                         uint8_t rxPort,
                         uint8_t txPort,
                         struct stat_s *stat,
                         struct flows_s *flows)
{
  struct packet_fields fields;
  struct rte_flow_error error;
  struct rte_flow *flow;
#ifdef _PERF_
   uint64_t hz = rte_get_timer_hz();
#endif

  if (flows->off_flow_count >= NUM_RTE_FLOWS) {
    printf("Flow table is full for port %u. Increase size in order to continue\n", rxPort);
    return 1;
  }

  fields.packet_type = 0;
  if (packet_decoder(rte_pktmbuf_mtod_offset(mb, uint8_t *, 0), &fields) != 0) {
    printf("Unsupported packet\n");
    return 1;
  }

  if (debug_print_out) {
    dump_decoded_packet(&fields, rxPort);
  }

  // Not incapsulated on ingress port, so drop it
  if (use_decap && rxPort == ingress_port && (fields.packet_type & RTE_PTYPE_TUNNEL_GTPU) == 0) return 0;

  // Packet must be an IP packet
  if (fields.packet_type <= 1) return 0;

  // Packet must have an known Layer4 protocol
  if ((fields.packet_type & RTE_PTYPE_L4_MASK) == 0) return 0;

  struct rte_flow_attr attr;
  struct rte_flow_item pattern[10];
  struct rte_flow_action actions[10];

  struct rte_flow_action_port_id id;
  struct rte_flow_action_meter meter;

  struct rte_flow_item_ipv4 ipv4;
  struct rte_flow_item_ipv6 ipv6;

  struct rte_flow_item_tcp tcp;
  struct rte_flow_item_udp udp;
  struct rte_flow_item_sctp sctp;

#ifdef MATCH_ON_INNER
  struct rte_flow_item_ipv4 ipv4_inner;
  struct rte_flow_item_ipv6 ipv6_inner;
  struct rte_flow_item_tcp tcp_inner;
  struct rte_flow_item_udp udp_inner;
  struct rte_flow_item_sctp sctp_inner;
#endif

  struct rte_flow_action_raw_decap decap;
  struct rte_flow_action_raw_encap encap;

  uint32_t actionCount;
  uint32_t patternCount;

  memset(&attr, 0, sizeof(attr));
  memset(&actions, 0, sizeof(actions));
  memset(&pattern, 0, sizeof(pattern));

  attr.ingress = 1;
  attr.priority = 1;

  actionCount = 0;
  patternCount = 0;

  switch (fields.packet_type & RTE_PTYPE_L3_MASK)
  {
  case RTE_PTYPE_L3_IPV4:
    {
      memset(&ipv4, 0, sizeof(struct rte_flow_item_ipv4));
      ipv4.hdr.src_addr = fields.match_ipv4_src;
      ipv4.hdr.dst_addr = fields.match_ipv4_dst;
      pattern[patternCount].type = RTE_FLOW_ITEM_TYPE_IPV4;
      pattern[patternCount].spec = &ipv4;
      pattern[patternCount].mask = &rte_flow_item_ipv4_mask;
      patternCount++;
      attr.group = 1;
      break;
    }
  case RTE_PTYPE_L3_IPV6:
    {
      memset(&ipv6, 0, sizeof(struct rte_flow_item_ipv6));
      memcpy(&ipv6.hdr.src_addr, fields.match_ipv6_src, 16);
      memcpy(&ipv6.hdr.dst_addr, fields.match_ipv6_dst, 16);
      pattern[patternCount].type = RTE_FLOW_ITEM_TYPE_IPV6;
      pattern[patternCount].spec = &ipv6;
      pattern[patternCount].mask = &rte_flow_item_ipv6_mask;
      patternCount++;
      attr.group = 2;
      break;
    }
  }

  switch (fields.packet_type & RTE_PTYPE_L4_MASK)
  {
  case RTE_PTYPE_L4_UDP:
    {
      memset(&udp, 0, sizeof(struct rte_flow_item_udp));
      udp.hdr.src_port = fields.match_udp_src;
      udp.hdr.dst_port = fields.match_udp_dst;
      pattern[patternCount].type = RTE_FLOW_ITEM_TYPE_UDP;
      pattern[patternCount].spec = &udp;
      pattern[patternCount].mask = &rte_flow_item_udp_mask;
      patternCount++;
      break;
    }
  case RTE_PTYPE_L4_TCP:
    {
      memset(&tcp, 0, sizeof(struct rte_flow_item_tcp));
      tcp.hdr.src_port = fields.match_tcp_src;
      tcp.hdr.dst_port = fields.match_tcp_dst;
      pattern[patternCount].type = RTE_FLOW_ITEM_TYPE_TCP;
      pattern[patternCount].spec = &tcp;
      pattern[patternCount].mask = &rte_flow_item_tcp_mask;
      patternCount++;
      break;
    }
  case RTE_PTYPE_L4_SCTP:
    {
      memset(&sctp, 0, sizeof(struct rte_flow_item_sctp));
      sctp.hdr.src_port = fields.match_sctp_src;
      sctp.hdr.dst_port = fields.match_sctp_dst;
      pattern[patternCount].type = RTE_FLOW_ITEM_TYPE_SCTP;
      pattern[patternCount].spec = &sctp;
      pattern[patternCount].mask = &rte_flow_item_sctp_mask;
      patternCount++;
      break;
    }
  }

#ifdef MATCH_ON_INNER
  if (fields.packet_type & RTE_PTYPE_TUNNEL_GTPU) {
    pattern[patternCount].type = RTE_FLOW_ITEM_TYPE_GTP;
    patternCount++;
  }

    switch (fields.packet_type & RTE_PTYPE_INNER_L3_MASK)
  {
  case RTE_PTYPE_INNER_L3_IPV4:
    {
      memset(&ipv4_inner, 0, sizeof(struct rte_flow_item_ipv4));
      ipv4_inner.hdr.src_addr = fields.match_ipv4_src_inner;
      ipv4_inner.hdr.dst_addr = fields.match_ipv4_dst_inner;
      pattern[patternCount].type = RTE_FLOW_ITEM_TYPE_IPV4;
      pattern[patternCount].spec = &ipv4_inner;
      pattern[patternCount].mask = &rte_flow_item_ipv4_mask;
      patternCount++;
      attr.group = 1;
      break;
    }
  case RTE_PTYPE_INNER_L3_IPV6:
    {
      memset(&ipv6_inner, 0, sizeof(struct rte_flow_item_ipv6));
      memcpy(&ipv6_inner.hdr.src_addr, fields.match_ipv6_src_inner, 16);
      memcpy(&ipv6_inner.hdr.dst_addr, fields.match_ipv6_dst_inner, 16);
      pattern[patternCount].type = RTE_FLOW_ITEM_TYPE_IPV6;
      pattern[patternCount].spec = &ipv6_inner;
      pattern[patternCount].mask = &rte_flow_item_ipv6_mask;
      patternCount++;
      attr.group = 2;
      break;
    }
  }

  switch (fields.packet_type & RTE_PTYPE_INNER_L4_MASK)
  {
  case RTE_PTYPE_INNER_L4_UDP:
    {
      memset(&udp_inner, 0, sizeof(struct rte_flow_item_udp));
      udp_inner.hdr.src_port = fields.match_udp_src_inner;
      udp_inner.hdr.dst_port = fields.match_udp_dst_inner;
      pattern[patternCount].type = RTE_FLOW_ITEM_TYPE_UDP;
      pattern[patternCount].spec = &udp_inner;
      pattern[patternCount].mask = &rte_flow_item_udp_mask;
      patternCount++;
      break;
    }
  case RTE_PTYPE_INNER_L4_TCP:
    {
      memset(&tcp_inner, 0, sizeof(struct rte_flow_item_tcp));
      tcp_inner.hdr.src_port = fields.match_tcp_src_inner;
      tcp_inner.hdr.dst_port = fields.match_tcp_dst_inner;
      pattern[patternCount].type = RTE_FLOW_ITEM_TYPE_TCP;
      pattern[patternCount].spec = &tcp_inner;
      pattern[patternCount].mask = &rte_flow_item_tcp_mask;
      patternCount++;
      break;
    }
  case RTE_PTYPE_INNER_L4_SCTP:
    {
      memset(&sctp_inner, 0, sizeof(struct rte_flow_item_sctp));
      sctp_inner.hdr.src_port = fields.match_sctp_src_inner;
      sctp_inner.hdr.dst_port = fields.match_sctp_dst_inner;
      pattern[patternCount].type = RTE_FLOW_ITEM_TYPE_SCTP;
      pattern[patternCount].spec = &sctp_inner;
      pattern[patternCount].mask = &rte_flow_item_sctp_mask;
      patternCount++;
      break;
    }
  }
#endif

  pattern[patternCount].type = RTE_FLOW_ITEM_TYPE_END;
  patternCount++;

  if (use_decap) {
    if (rxPort == ingress_port) {
      decap.size = sizeof(struct decap_encap_s);
      decap.data = (uint8_t *)&decap0Buffer;
      encap.size = sizeof(struct encap_decap_s);
      encap.data = (uint8_t *)&encap0Buffer;
    }
    else {
      decap.size = sizeof(struct encap_decap_s);
      decap.data = (uint8_t *)&decap1Buffer;
      encap.size = sizeof(struct decap_encap_s);
      encap.data = (uint8_t *)&encap1Buffer;
    }

    actions[actionCount].type = RTE_FLOW_ACTION_TYPE_RAW_DECAP;
    actions[actionCount].conf = &decap;
    actionCount++;

    actions[actionCount].type = RTE_FLOW_ACTION_TYPE_RAW_ENCAP;
    actions[actionCount].conf = &encap;
    actionCount++;
  }

  id.id = txPort;
  actions[actionCount].type = RTE_FLOW_ACTION_TYPE_PORT_ID;
  actions[actionCount].conf = &id;
  actionCount++;

  if (use_meter) {
    meter.mtr_id = meter_mtr_id[rxPort];
    actions[actionCount].type = RTE_FLOW_ACTION_TYPE_METER;
    actions[actionCount].conf = &meter;
    actionCount++;
  }

  actions[actionCount].type = RTE_FLOW_ACTION_TYPE_END;
  actionCount++;

#ifdef _PERF_
  tsc1 = rte_rdtsc();
#endif
  flow = rte_flow_create(rxPort, &attr, pattern, actions, &error);
#ifdef _PERF_
  tsc2 = rte_rdtsc();
  counter++;
  diff += tsc2-tsc1;
  double div = (double)(diff/counter)/(double)hz;
  printf("Time: %f %16llu (%llu, %llu)\n", div, (long long unsigned int)(diff/counter), (long long unsigned int)counter, (long long unsigned int)(tsc2-tsc1));
#endif
  if (flow == NULL) {
    printf("ERROR (%s): Port: %u, Group: %u, Packet type %X, MSG: %s\n", __func__, rxPort, attr.group, mb->packet_type, error.message);
    return 1;
  }

  flows->off_flows[flows->off_flow_count] = flow;
  flows->off_flow_count++;
  stat->numFilter++;

  return 0;
}
