#ifndef __FILTER_H__
#define __FILTER_H__

#include <stdint.h>

#define NO_PORTS   2
#define MAX_QUEUES 4
#define NUM_RTE_FLOWS 10000000

#define MAX_DECAP_SIZE 128
#define MAX_ENCAP_SIZE 128

#define IPV4_ADDRESS(a) ((const char *)&a)[0] & 0xFF, \
                        ((const char *)&a)[1] & 0xFF, \
                        ((const char *)&a)[2] & 0xFF, \
                        ((const char *)&a)[3] & 0xFF

#define IPV6_ADDRESS(a) (unsigned int)(a[0] & 0xFF), \
                        (unsigned int)(a[1] & 0xFF), \
                        (unsigned int)(a[2] & 0xFF), \
                        (unsigned int)(a[3] & 0xFF), \
                        (unsigned int)(a[4] & 0xFF), \
                        (unsigned int)(a[5] & 0xFF), \
                        (unsigned int)(a[6] & 0xFF), \
                        (unsigned int)(a[7] & 0xFF), \
                        (unsigned int)(a[8] & 0xFF), \
                        (unsigned int)(a[9] & 0xFF), \
                        (unsigned int)(a[10] & 0xFF), \
                        (unsigned int)(a[11] & 0xFF), \
                        (unsigned int)(a[12] & 0xFF), \
                        (unsigned int)(a[13] & 0xFF), \
                        (unsigned int)(a[14] & 0xFF), \
                        (unsigned int)(a[15] & 0xFF)


struct packet_fields {
  uint32_t vlan_id;
  struct rte_flow_item_ipv4 ipv4;
  union {
    uint32_t match_ipv4_src;
    uint8_t match_ipv6_src[16];
  };
  union {
    uint32_t match_ipv4_dst;
    uint8_t match_ipv6_dst[16];
  };
  union {
    uint16_t match_udp_src;
    uint16_t match_tcp_src;
    uint16_t match_sctp_src;
  };
  union {
    uint16_t match_udp_dst;
    uint16_t match_tcp_dst;
    uint16_t match_sctp_dst;
  };
  uint32_t encap_gtp_teid;
  uint8_t inner_protocol;
  union {
    uint32_t match_ipv4_src_inner;
    uint8_t match_ipv6_src_inner[16];
  };
  union {
    uint32_t match_ipv4_dst_inner;
    uint8_t match_ipv6_dst_inner[16];
  };
  union {
    uint16_t match_udp_src_inner;
    uint16_t match_tcp_src_inner;
    uint16_t match_sctp_src_inner;
  };
  union {
    uint16_t match_udp_dst_inner;
    uint16_t match_tcp_dst_inner;
    uint16_t match_sctp_dst_inner;
  };
  uint32_t packet_type;
};

struct stat_s {
  volatile uint64_t numRxPackets;
  volatile uint64_t numFilter;
};

struct flows_s {
  struct rte_flow *off_flows[NUM_RTE_FLOWS];
  uint64_t off_flow_count;
};

struct flows_stat_s {
  struct flows_s f[NO_PORTS][MAX_QUEUES];
};

struct decap_encap_s {
    struct rte_ether_hdr eth;
    struct rte_vlan_hdr vlan;
    struct rte_ipv4_hdr ipv4;
    struct rte_udp_hdr udp;
    struct rte_gtp_hdr gtp;
    struct rte_gtp_hdr_ext_word ext_word;
    struct gtp_pdu_session_container {
      uint8_t ext_hdr_len;
#if RTE_BYTE_ORDER == RTE_BIG_ENDIAN
      uint8_t type:4;
      uint8_t spare:4;
      uint8_t ppp:1;
      uint8_t rqi:1;
      uint8_t qfi:6;
#else
      uint8_t spare:4;
      uint8_t type:4;
      uint8_t qfi:6;
      uint8_t rqi:1;
      uint8_t ppp:1;
#endif
      uint8_t next_ext;
    } gtp_psc;
} __rte_packed;

struct encap_decap_s {
    struct rte_ether_hdr eth;
    struct rte_vlan_hdr vlan;
} __rte_packed;

int offload_filter_setup(struct rte_mbuf *mb, uint8_t rxPort, uint8_t txPort, struct stat_s *stat, struct flows_s *flows);
void set_eth_mac_address(uint16_t port_id, struct rte_ether_addr *mac_addr);
void set_vlan(uint16_t port, uint16_t tci);

#endif /* __FILTER_H__ */
