This file contains instructions on how to run traffic to demo application.

There are 2 supported options on how to send traffic to demo application:
- re-play prepared PCAP file;
- use TREX traffic generator.

For correct work of these methods the'Napatech Link-Capture™' Software and the 'Napatech libpcap'
is needed to be installed.
Detail instruction how on how to install necessary software:
https://docs.napatech.com/r/Link-CaptureTM-Software-Getting-Started-Guide/Installing-Napatech-Link-CaptureTM-Software

# 1. Ntpcap reply tool

This section shows how to run already prepared pcap files for testing.

Use 'ntpcap_replay' application to run traffic from port 0 of
the inserted NT NIC of Node 1 with specified pcap file.
The packets are transmitted to the pcap port specified by the -p option.

Using the following command on Node 1:
```bash
/opt/napatech3/bin/ntpcap_replay -f ./test.pcap -p 0
```

# 2. TRex configuration for inline UPF demo

This section describes how to setup TREX traffic generator to send uplink/downlink UPF traffic
(with and without GTP headers).

Uplink/downlink traffic structure is hard-coded inside 'trex_inline_upf_demo.py'
which is responsible for traffic generator.

In order to change pre-defined traffic structure end user might modify
'trex_inline_upf_demo.py' client script using TREX STL API.

## 2.1 TRex server description

__Hardware__:

* **CPU**:
   Intel(R) Xeon(R) Gold 6209U CPU @ 2.10GHz, 2100 MHz (40 cores)

* **Network**:
  Broadcom NetXtreme BCM5720 Gigabit Ethernet PCIe
  Napatech A/S NT200A02 Network Adapter

* **Memory**:
  RAM - 187GB (32GB HugePages)

* **FPGA**
  Capture 100G

## 2.2 Traffic flow description

N3 traffic flow

```log
TRex (GTPv1U)  ----->   0 port | Inline UPF demo | port 1(MBR) ----> decapsulated traffic
```

N6 traffic flow

```log
TRex           ----->   1 port | Inline UPF demo | port 0(MBR) ----> encapsulated traffic
```

## 3. Build TREX

This section describes how to build TREX.

### 3.1 Get source code

```bash
git clone https://github.com/cisco-system-traffic-generator/trex-core.git
```

### 3.2 Configure

```bash
cd trex-core/linux_dpdk
./b configure --no-mlx=NO_MLX --with-ntacc
```

```log
Setting top to                           : /usr/project/trex-core
Setting out to                           : /usr/project/trex-core/linux_dpdk/build_dpdk
Checking for program 'gcc, cc'           : /usr/bin/gcc
Checking for program 'ar'                : /usr/bin/ar
Checking for program 'g++, c++'          : /usr/bin/g++
Checking for program 'ar'                : /usr/bin/ar
Checking for program 'ldd'               : /usr/bin/ldd
Checking for library z                   : yes
Build sanitized images (GCC >= 4.9.0)    : no
Checking for NTAPI                       : Found needed NTAPI library
'configure' finished successfully (0.408s)
```

### 3.3 Build TRex

Refer to [NapaTech documentation](https://docs.napatech.com/r/Installation-and-Use-of-Napatech-LinkTM-Capture-Software-for-Intel-PAC-with-Intel-Arria-10-GX-FPGA/TRex)
for details

```bash
cd trex-core/linux_dpdk
./b
```

```log
Waf: Entering directory `/usr/builds/trex-core.github.rsync/linux_dpdk/build_dpdk'
 Info: Using external libverbs.
 update version files
 ...
 [1206/1210] Compiling version.c
 [1207/1210] Linking build_dpdk/linux_dpdk/_t-rex-64-debug-o
 [1208/1210] Linking build_dpdk/linux_dpdk/_t-rex-64-debug
 [1209/1210] Linking build_dpdk/linux_dpdk/_t-rex-64-o
 [1210/1210] Linking build_dpdk/linux_dpdk/_t-rex-64
 Waf: Leaving directory `/usr/builds/trex-core.github.rsync/linux_dpdk/build_dpdk'
*** generating softlinks ***
'build' finished successfully (15.621s)
```

# 4. Environment preparation

This section describes how to prepare environment for TREX run.

## 4.1 Configure nt3gd driver

### Change the number of RX and TX buffers in ntservice.ini from 4 to 128:

```bash
cd /opt/napatech3/config/
vim ntservice.ini
```


```log
HostBuffersRx = [128,200,-1]                # [x1, x2, x3], ...
HostBuffersTx = [128,200,-1]                # [x1, x2, x3], ...
```

## 4.2 Start NT services

```bash
sudo /opt/napatech3/bin/ntstart.sh
Loading nt3gd driver                                        [Done]
Creating driver device file                                 [Done]
Loading nt3gd_netdev driver                                 [Done]
Creating driver device file                                 [Done]
Starting NTService (this may take a while)                  [Done]
```

## 4.3 Use dpdk_setup_ports.py to create the TREX config file

```bash
./dpdk_setup_ports.py -i
```

```log
By default, IP based configuration file will be created. Do you want to use MAC based config? (y/N)y
+-----+------+-----------+-------------------+-----------------------------------------+--------+----------+----------+
| ID  | NUMA |    PCI    |        MAC        |                  Name                   | Driver | Linux IF |  Active  |
+=====+======+===========+===================+=========================================+========+==========+==========+
| 0   | 0    | 04:00.0   | 2c:ea:7f:ca:a9:d3 | NetXtreme BCM5720 Gigabit Ethernet PCIe | tg3    | eno1     | *Active* |
+-----+------+-----------+-------------------+-----------------------------------------+--------+----------+----------+
| 1   | 0    | 04:00.1   | 2c:ea:7f:ca:a9:d4 | NetXtreme BCM5720 Gigabit Ethernet PCIe | tg3    | eno2     |          |
+-----+------+-----------+-------------------+-----------------------------------------+--------+----------+----------+
| 2/0 | 0    | 65:00.0/0 | 00:0d:e9:07:18:16 | NT200A02 Network Adapter                | nt3gd  |          |          |
+-----+------+-----------+-------------------+-----------------------------------------+--------+----------+----------+
| 2/1 | 0    | 65:00.0/1 | 00:0d:e9:07:18:17 | NT200A02 Network Adapter                | nt3gd  |          |          |
+-----+------+-----------+-------------------+-----------------------------------------+--------+----------+----------+
Please choose an even number of interfaces from the list above, either by ID, PCI or Linux IF
Stateful will use order of interfaces: Client1 Server1 Client2 Server2 etc. for flows.
Stateless can be in any order.
Enter list of interfaces separated by space (for example: 1 3) : 2/0 2/1

For interface 2, assuming loopback to its dual interface 2.
Destination MAC is 00:0d:e9:07:18:17. Change it to MAC of DUT? (y/N).N
For interface 2, assuming loopback to its dual interface 2.
Destination MAC is 00:0d:e9:07:18:16. Change it to MAC of DUT? (y/N).N
Print preview of generated config? (Y/n)Y
### Config file generated by dpdk_setup_ports.py ###

- version: 2
  interfaces: ['65:00.0/0', '65:00.0/1']
  port_info:
      - dest_mac: 00:0d:e9:07:18:17 # MAC OF LOOPBACK TO IT'S DUAL INTERFACE
        src_mac:  00:0d:e9:07:18:16
      - dest_mac: 00:0d:e9:07:18:16 # MAC OF LOOPBACK TO IT'S DUAL INTERFACE
        src_mac:  00:0d:e9:07:18:17

  platform:
      master_thread_id: 0
      latency_thread_id: 39
      dual_if:
        - socket: 0
          threads: [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38]


Save the config to file? (Y/n)Y
Default filename is /etc/trex_cfg.yaml
Press ENTER to confirm or enter new file:
File /etc/trex_cfg.yaml already exist, overwrite? (y/N)y
Saved to /etc/trex_cfg.yaml.
```

# 5. Start of TRex server (Terminal 1)

```bash
cd trex-core/scripts
./t-rex-64 -i -c 10 --stl
```

# 6. Start of Trex client (Terminal 1)

```bash
cd trex-core
export PYTHONPATH=$PWD/scripts/automation/trex_control_plane/interactive
```

# 7. Traffic Generation

Copy TRex client scripts `trex_inline_upf_demo_zerodrop.py` and `trex_inline_upf_demo_zerodrop_NAT.py` to the Trex server

```bash
scp <upf_user>@<upf_ip>:<workspace_path>/ntoss.git/examples/inline_upf_demo/traffic_gen_tools_conf/trex_inline_upf_demo_zerodrop.py
scp <upf_user>@<upf_ip>:<workspace_path>/ntoss.git/examples/inline_upf_demo/traffic_gen_tools_conf/trex_inline_upf_demo_zerodrop_NAT.py
```

__Run traffic generation__

```
python3 trex_inline_upf_demo_zerodrop.py
```

__Run traffic generation with NAT__

>NOTE: inline upf demo application must be run with option '-m 13312 -- -n'

```
python3 trex_inline_upf_demo_zerodrop_NAT.py
```
__The difference between the modified tests (zerodrop) and the previously written ones.__

The tests that were developed faced challenges related to checksums and drop events during flow learning and transmitting. However, in the zerodrop version of the tests, no drops events occur when performing flow learning at a low rate and when demonstrating traffic transmission at full rate.

__Known issues__

The use of the STLVmFlowVar API for TEID introduces variability in the sequence of generation during execution, thereby posing a challenge to ensuring reliable and accurate execution. Therefore, we are encountering an issue where we are unable to both learn and transmit identical traffic.
